<?php
/**
 * @package     Joomla.Site
 * @subpackage  mod_totalmenu
 *
 * @copyright   Copyright (C) 2005 - 2018 Open Source Matters, Inc. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 */

defined('_JEXEC') or die;

use Joomla\Archive\Archive;
use Joomla\CMS\Factory;
use Joomla\CMS\Filesystem\File;
use Joomla\CMS\Filesystem\Folder;
use Joomla\CMS\Helper\ModuleHelper;
use Joomla\CMS\Language\Text;
use Joomla\Registry\Registry;
use Jtotal\Module\Totalmenu\Site\Helper\TotalmenuProcessCss;

/**
 * Helper for mod_totalmenu
 *
 * @since  1.5
 */
class ModtotalmenuHelper
{

    private static $customcode = array();
    private static $a_customcode = array();
    private static $c_customcode = array();

    //caret
    public static $caret_desktop_on;
    public static $caret_mobile_on;
    public static $caret_current_on;


    /**
     * Get a list of the menu items.
     *
     * @param   \Joomla\Registry\Registry  &$params  The module options.
     *
     * @return  array
     *
     * @since   1.5
     */
    /*
    public static function getList(&$params, $items_menu = '')
    {
        $tmmode = $params->get('tmmode', 'menu');

        $app = Factory::getApplication();
        $menu = $app->getMenu();

        // Get active menu item
        $base = self::getBase($params);
        $levels = Factory::getUser()->getAuthorisedViewLevels();
        asort($levels);

        // Compose cache key
        $cacheKey = 'menu_items' . $params . implode(',', $levels) . '.' . $base->id;

        $cache = Factory::getContainer()->get(CacheControllerFactoryInterface::class)
                ->createCacheController('output', ['defaultgroup' => 'mod_menu']);

        if ($cache->contains($cacheKey)) {
            $items = $cache->get($cacheKey);
        } else {

            $path = $base->tree;
            $start = (int) $params->get('startLevel', 1);
            $end = (int) $params->get('endLevel', 0);
            $showAll = $params->get('showAllChildren', 1);
            //$items          = $menu->getItems('menutype', $params->get('menutype'));
            $hidden_parents = array();
            $lastitem = 0;

            $superItems = array();

            //SUPERPARENT MENU
            $superMenuItem = new MenuItem();
            $superMenuItem->id = 0;
            $superMenuItem->title = $items_menu;
            $superMenuItem->level = '1';
            $superMenuItem->type = 'superparent';

            //MENU
            if ($tmmode == 'menu') {
                $items = array();
                $items = $menu->getItems('menutype', $params->get('menutype'));
            }

            //MENUES
            if ($tmmode == 'menues') {

                $items[] = $superMenuItem;

                foreach ($menu->getItems('menutype', $items_menu) as $key => $menu) {

                    //if highest level in tree is not supermenu id...
                    if ($menu->tree[0] != $superMenuItem->id) {
                        array_unshift($menu->tree, $superMenuItem->id);
                    }

                    $menu->level = (string) ((int) $menu->level + 1);
                    //$menu->parent = (string) ((int) $menu->level + 1);

                    $items[] = $menu;
                }
            }

            if ($items) {

                $inputVars = $app->getInput()->getArray();
 

                foreach ($items as $i => $item) {

                    //$inputVars = $app->getInput()->getArray();

                    //var_dump($item);
                    //SUPERPARENT

                    $item->parent = false;
                    $itemParams = $item->getParams();

                    //IS PARENT?
                    if (
                        isset($items[$lastitem]) &&
                        $items[$lastitem]->id == $item->parent_id &&
                        $itemParams->get('menu_show', 1) == 1
                    ) {
                        $items[$lastitem]->parent = true;
                    }

                    if (($start && $start > $item->level)
                        || ($end && $item->level > $end)
                        || (!$showAll && $item->level > 1 && !\in_array($item->parent_id, $path))
                        || ($start > 1 && !\in_array($item->tree[$start - 2], $path))) {
                        unset($items[$i]);

                        continue;
                    }

                    // Exclude item with menu item option set to exclude from menu modules
                    if (($itemParams->get('menu_show', 1) == 0) || in_array($item->parent_id, $hidden_parents)) {
                        $hidden_parents[] = $item->id;
                        unset($items[$i]);
                        continue;
                    }

                    $item->current = true;

                    foreach ($item->query as $key => $value) {
                        if (!isset($inputVars[$key]) || $inputVars[$key] !== $value) {
                            $item->current = false;
                            break;
                        }
                    }

                    $item->deeper = false;
                    $item->shallower = false;
                    $item->level_diff = 0;

                    if (isset($items[$lastitem])) {
                        $items[$lastitem]->deeper = ($item->level > $items[$lastitem]->level);
                        $items[$lastitem]->shallower = ($item->level < $items[$lastitem]->level);
                        $items[$lastitem]->level_diff = ($items[$lastitem]->level - $item->level);
                    }

                    $lastitem = $i;
                    $item->active = false;
                    $item->flink = $item->link;

                    // Reverted back for CMS version 2.5.6
                    switch ($item->type) {

                        case 'separator':
                            break;

                        case 'heading':
                            // No further action needed.
                            break;

                        case 'url':
                            if ((strpos($item->link, 'index.php?') === 0) && (strpos($item->link, 'Itemid=') === false)) {
                                // If this is an internal Joomla link, ensure the Itemid is set.
                                $item->flink = $item->link . '&Itemid=' . $item->id;
                            }
                            break;

                        case 'alias':
                            $item->flink = 'index.php?Itemid=' . $itemParams->get('aliasoptions');

                            // Get the language of the target menu item when site is multilingual
                            if (Multilanguage::isEnabled()) {
                                $newItem = Factory::getApplication()->getMenu()->getItem((int) $itemParams->get('aliasoptions'));

                                // Use language code if not set to ALL
                                if ($newItem != null && $newItem->language && $newItem->language !== '*') {
                                    $item->flink .= '&lang=' . $newItem->language;
                                }
                            }
                            break;

                        case 'superparent':
                            $item->flink = '#';
                            break;

                        default:
                            $item->flink = 'index.php?Itemid=' . $item->id;
                            break;
                    }

                    if ((strpos($item->flink, 'index.php?') !== false) && strcasecmp(substr($item->flink, 0, 4), 'http')) {
                        $item->flink = Route::_($item->flink, true, $itemParams->get('secure'));
                    } else {
                        $item->flink = Route::_($item->flink);
                    }

                    // We prevent the double encoding because for some reason the $item is shared for menu modules and we get double encoding
                    // when the cause of that is found the argument should be removed
                    $item->title = htmlspecialchars($item->title, ENT_COMPAT, 'UTF-8', false);
                    $item->menu_icon = htmlspecialchars($itemParams->get('menu_icon_css', ''), ENT_COMPAT, 'UTF-8', false);
                    $item->anchor_css = htmlspecialchars($itemParams->get('menu-anchor_css', ''), ENT_COMPAT, 'UTF-8', false);
                    $item->anchor_title = htmlspecialchars($itemParams->get('menu-anchor_title', ''), ENT_COMPAT, 'UTF-8', false);
                    $item->anchor_rel = htmlspecialchars($itemParams->get('menu-anchor_rel', ''), ENT_COMPAT, 'UTF-8', false);
                    $item->menu_image = htmlspecialchars($itemParams->get('menu_image', ''), ENT_COMPAT, 'UTF-8', false);
                    $item->menu_image_css = htmlspecialchars($itemParams->get('menu_image_css', ''), ENT_COMPAT, 'UTF-8', false);
                }

                if (isset($items[$lastitem])) {
                    $items[$lastitem]->deeper = (($start ?: 1) > $items[$lastitem]->level);
                    $items[$lastitem]->shallower = (($start ?: 1) < $items[$lastitem]->level);
                    $items[$lastitem]->level_diff = ($items[$lastitem]->level - ($start ?: 1));
                }
            }

            if (isset($key)) {
                $cache->store($items, $key);
            }
        }

        return $items;
    }
    */

    /*
    public static function arrayControlV2($treeControl, $item, $year, $month, $day, $params)
    {

        $year_on = in_array('year', $params->get('tmcategory_date_parent', array()));
        $month_on = in_array('month', $params->get('tmcategory_date_parent', array()));
        $day_on = in_array('day', $params->get('tmcategory_date_parent', array()));
        $cat_on = in_array('cat', $params->get('tmcategory_date_parent', array()));
        $art_on = in_array('art', $params->get('tmcategory_date_parent', array()));

        $retorno = array();

        if ($year_on) {
            $retorno[$year];
        }

        //$treeControl[$year][$month][$day][$item->catid][$item->id];

    }


    public static function getBase(&$params)
    {
        // Get base menu item from parameters
        if ($params->get('base')) {
            $base = Factory::getApplication()->getMenu()->getItem($params->get('base'));
        } else {
            $base = false;
        }

        // Use active menu item if no base found
        if (!$base) {
            $base = self::getActive($params);
        }

        return $base;
    }


    public static function getActive(&$params = '')
    {
        $menu = Factory::getApplication()->getMenu();

        return $menu->getActive() ?: self::getDefault();
    }
    */
    /**
     * Get default menu item (home page) for current language.
     *
     * @return  object
     */
    /*
    public static function getDefault()
    {
        $menu = Factory::getApplication()->getMenu();
        $lang = Factory::getLanguage();

        // Look for the home menu
        if (JLanguageMultilang::isEnabled()) {
            return $menu->getDefault($lang->getTag());
        } else {
            return $menu->getDefault();
        }
    }
    */

    /**
     * Get Fontawesome classes.
     *
     * @return  object
     */
    /*
    public static function getFa($item)
    {

        //var_dump($item);
        $classes_array = array_map('trim', array_filter(explode(' ', $item->anchor_css)));
        $classes_no_fa = array();
        $classes_fa = array();
        foreach ($classes_array as $key => $class) {

            //check fa class
            if (
                $class == 'fas' ||
                $class == 'far' ||
                $class == 'fal' ||
                $class == 'fab' ||
                $class == 'fad'
            ) {
                $classes_fa[] = $class;

            } elseif (substr($class, 0, 3) === 'tm-') {
                $classes_fa[] = str_replace("tm-", "", $class);
                //$classes_fa[] = str_replace("","tm-",$class);

            } elseif (substr($class, 0, 3) === 'fa-') {
                $classes_fa[] = $class;
            } else {
                $classes_no_fa[] = $class;
            }
        }

        $classes_fa = implode(' ', $classes_fa);
        //$fa_icon = $classes_fa ? '<i class="tm_fa_menu '.$classes_fa.'"></i>' : '';
        //$custom_classes_from_menu = implode(' ', $classes_no_fa);

        $tmfa_array = array('fa' => $classes_fa, 'nofa' => $classes_no_fa);

        return $tmfa_array;
    }
    */

    public static function getParamsAjax()
    {
        $jinput = Factory::getApplication()->input;
        $moduleid = $jinput->get('id', '');

        $module = ModuleHelper::getModuleById($moduleid);

        $params = new Registry($module->params);

        $params->set('jtotal_email', 'hidden');
        $params->set('jtotal_key', 'hidden');

        return $params;
    }

    public static function importZipAjax()
    {

        $jinput = Factory::getApplication()->input;
        $moduleid = $jinput->get('id', '');
        $token = $jinput->get('token', '');

        //return $_FILES;
        if (!isset($_FILES['importzip']['tmp_name'])) {
            return false;
        }

        $archive = new Archive;
        $archive->getAdapter('zip')->extract($_FILES['importzip']['tmp_name'], JPATH_SITE . '/tmp/totalmenu_' . $token . '_' . $moduleid);

        $params_from_file = json_decode(file_get_contents(JPATH_SITE . '/tmp/totalmenu_' . $token . '_' . $moduleid . '/jform_params_exportData.json'));

        $params = new Registry($params_from_file->params);
        $params->set('exportData', 0);


        $table = JTable::getInstance('module');
        $table->load($moduleid);
        $table->bind(array('params' => json_encode($params)));

        if (!$table->check()) {
            //$this->setError('lastcreatedate: check: ' . $table->getError());
            return 'lastcreatedate: check: ' . $table->getError();
        }
        if (!$table->store()) {
            return 'lastcreatedate: store: ' . $table->getError();
        }

        File::delete(JPATH_SITE . '/tmp/totalmenu_' . $token . '_' . $moduleid . '/jform_params_exportData.json');

        return true;
    }

    public static function deleteCacheAjax()
    {

        $jinput = Factory::getApplication()->input;
        $moduleid = $jinput->get('id', '');
        $token = $jinput->get('token', '');

        if (Folder::exists(JPATH_SITE . '/cache/mod_totalmenu')) {
            Folder::delete(JPATH_SITE . '/cache/mod_totalmenu');
            return true;
        }
        return JPATH_SITE . '/cache/mod_totalmenu';
        return true;
    }

    public static function getDeviceAjax()
    {

        ///////////////////
        //  CHECK FW //
        ///////////////////
        if (is_file(JPATH_PLUGINS . '/system/jtframework/autoload.php')) {
            require_once JPATH_PLUGINS . '/system/jtframework/autoload.php';
        } else {
            $app = Factory::getApplication();
            $app->enqueueMessage(Text::_('JT_TOTALMENU_ERROR'), 'error');
            return '!!!!ERROR ' . Text::_('JT_TOTALMENU_ERROR');
        }

        $jinput = Factory::getApplication()->input;
        $moduleid = $jinput->get('moduleid', '');
        $device = $jinput->get('device', '');
        $module = ModuleHelper::getModuleById($moduleid);

        $params = new Registry($module->params);

        $unique = htmlspecialchars($params->get('dx_uniquename').$moduleid);

        if ($device == 'desktop') {
            $css = TotalmenuProcessCss::getDesktopCSS($params, $unique);
        }

        if ($device == 'mobile') {
            $css = TotalmenuProcessCss::getMobileCSS($params, $unique);
        }

        if ($params->get('tm_debug', 0)) {
            return $css;
        }

        return self::$a_customcode;

        return $device;
    }



    /******************************************************/
    /************            VARIABLES            *********/
    /******************************************************/

    /*
    private static function getVariables($params, $context = '')
    {

        //carets
        $tm_mobile_caret = new Registry;
        $tm_mobile_caret->loadString(json_encode($params->get('tm_mobile_caret')));
        self::$caret_mobile_on = $tm_mobile_caret->get('tm_caret_open', 'fa-arrow-right');

        $tm_desktop_caret = new Registry;
        $tm_desktop_caret->loadString(json_encode($params->get('tm_desktop_caret')));
        self::$caret_desktop_on = $tm_desktop_caret->get('tm_caret_open', 'fa-arrow-right');

        if ($context == 'mobile') {
            $custom = $params->get('tm_mobile_custom', (object) array());
            self::$caret_current_on = self::$caret_mobile_on;

        } else {
            $custom = $params->get('tm_desktop_custom', (object) array());
            self::$caret_current_on = self::$caret_desktop_on;
        }

        $desktop_base = $params->get('tm_desktop_base', 'horizontal');
        $mobile_base = $params->get('tm_mobile_base', 'horizontal');

        //$atribs_array = json_decode($custom);
        $atribs_obj = $custom;

        $custom_css = '';

        //prevent errors
        $custom_css = '
				$lvl0_bg: transparent !default;
				$lvl0_bg_open:transparent !default;
				$lvl0_bg_current:transparent !default;
				$lvl0_size:1rem !default;
				$lvl0_color:#333 !default;
				$lvl0_current:#111 !default;
				$lvl0_border_size:0 !default;
				$lvl0_active_border_size:0 !default;
				$lvl0_border_color:transparent !default;
				$lvl0_active_border_color:transparent !default;
				$lvl0_padding:10px 6px 6px 6px !default;
				$lvl0_active_padding:10px 6px 6px 6px !default;
				$lvl0_br:0px 0px 0px 0px !default;
				$lvl0_a_br:0px 0px 0px 0px !default;

			';

        //self::$customcode['lvl'.$key] = $code;
        //self::$a_customcode['lvl'.$key] = $a_code;
        //self::$c_customcode['lvl'.$key] = $c_code;

        foreach ($atribs_obj as $key => $table) {

            //KEY = tm_desktop_custom0, tm_desktop_custom1, etc

            foreach ($table as $propkey => $prop) {

                if (!is_object($prop)) {
                    $prop = json_decode($prop);
                }
                $prop = new Registry($prop);

                //var_dump($prop);

                //backgroud
                if ($prop->get('tm_background_on', 0)) {
                    $custom_css .= '$' . $key . '_bg:' . $prop->get('tm_background', '') . ';';
                    $custom_css .= '$' . $key . '_a_bg:' . $prop->get('a_tm_background', '') . ';';
                    $custom_css .= '$' . $key . '_c_bg:' . $prop->get('c_tm_background', '') . ';';
                }

                //color
                if ($prop->get('tm_color_on', 0)) {
                    $custom_css .= '$' . $key . '_color:' . $prop->get('tm_color', '') . ';';
                    $custom_css .= '$' . $key . '_a_color:' . $prop->get('a_tm_color', '') . ';';
                    $custom_css .= '$' . $key . '_c_color:' . $prop->get('c_tm_color', '') . ';';
                }

                //size
                if ($prop->get('tm_size_on', 0)) {
                    $custom_css .= '$' . $key . '_size:' . $prop->get('tm_size', '') . 'px;';
                    $custom_css .= '$' . $key . '_a_size:' . $prop->get('a_tm_size', '') . 'px;';
                    $custom_css .= '$' . $key . '_c_size:' . $prop->get('c_tm_size', '') . 'px;';
                }

                //BORDER width
                if ($prop->get('tm_border_width_on', 0)) {

                    $b_width = str_replace(',', 'px ', $prop->get('tm_border_width', '0,0,0,0')) . 'px';
                    $a_b_width = str_replace(',', 'px ', $prop->get('a_tm_border_width', '0,0,0,0')) . 'px';
                    $c_b_width = str_replace(',', 'px ', $prop->get('c_tm_border_width', '0,0,0,0')) . 'px';

                    $custom_css .= '$' . $key . '_border_width:' . $b_width . ';';
                    $custom_css .= '$' . $key . '_a_border_width:' . $a_b_width . ';';
                    $custom_css .= '$' . $key . '_c_border_width:' . $c_b_width . ';';
                }

                //BORDER color
                if ($prop->get('tm_border_color_on', 0)) {

                    $b_color = $prop->get('tm_border_color', 'rgba(0,0,0,0');
                    $a_b_color = $prop->get('a_tm_border_color', 'rgba(0,0,0,0');
                    $c_b_color = $prop->get('c_tm_border_color', 'rgba(0,0,0,0');

                    $custom_css .= '$' . $key . '_border_color:' . $b_color . ';';
                    $custom_css .= '$' . $key . '_a_border_color:' . $a_b_color . ';';
                    $custom_css .= '$' . $key . '_c_border_color:' . $c_b_color . ';';
                }

                //PADDING
                if ($prop->get('tm_padding_on', 0)) {

                    $padding = str_replace(',', 'px ', $prop->get('tm_padding', '0,0,0,0')) . 'px';
                    $a_padding = str_replace(',', 'px ', $prop->get('a_tm_padding', '0,0,0,0')) . 'px';
                    $c_padding = str_replace(',', 'px ', $prop->get('c_tm_padding', '0,0,0,0')) . 'px';

                    $custom_css .= '$' . $key . '_padding:' . $padding . ';';
                    $custom_css .= '$' . $key . '_a_padding:' . $a_padding . ';';
                    $custom_css .= '$' . $key . '_c_padding:' . $c_padding . ';';
                }

                //var_dump($prop);

                //BORDER radius
                if ($prop->get('tm_border_radius_on', 0)) {

                    $b_radius = str_replace(',', 'px ', $prop->get('tm_border_radius', '0,0,0,0')) . 'px';
                    $b_radius_array = explode(',', $prop->get('tm_border_radius', '0,0,0,0'));

                    $a_b_radius = str_replace(',', 'px ', $prop->get('a_tm_border_radius', '0,0,0,0')) . 'px';
                    $a_b_radius_array = explode(',', $prop->get('a_tm_border_radius', '0,0,0,0'));

                    $c_b_radius = str_replace(',', 'px ', $prop->get('c_tm_border_radius', '0,0,0,0')) . 'px';
                    $c_b_radius_array = explode(',', $prop->get('c_tm_border_radius', '0,0,0,0'));

                    $custom_css .= '$' . $key . '_border_radius:' . $b_radius . ';';
                    //$custom_css .= '$'.$key.'_border_radius_first_li:'.$b_radius_array[0].'px '.$b_radius_array[1].'px 0px 0px;';
                    //$custom_css .= '$'.$key.'_border_radius_last_li: 0px 0px '.$b_radius_array[2].'px '.$b_radius_array[3].'px;';

                    $custom_css .= '$' . $key . '_a_border_radius:' . $a_b_radius . ';';
                    //$custom_css .= '$'.$key.'_a_border_radius_first_li:'.$a_b_radius_array[0].'px '.$a_b_radius_array[1].'px 0px 0px;';
                    //$custom_css .= '$'.$key.'_a_border_radius_last_li: 0px 0px '.$a_b_radius_array[2].'px '.$a_b_radius_array[3].'px;';

                    $custom_css .= '$' . $key . '_c_border_radius:' . $c_b_radius . ';';
                    //$custom_css .= '$'.$key.'_c_border_radius_first_li:'.$c_b_radius_array[0].'px '.$c_b_radius_array[1].'px 0px 0px;';
                    //$custom_css .= '$'.$key.'_c_border_radius_last_li: 0px 0px '.$c_b_radius_array[2].'px '.$c_b_radius_array[3].'px;';

                }

                //CODE
                if ($prop->get('tm_code_on', 0)) {

                    $code = $prop->get('tm_code', '');
                    $a_code = $prop->get('a_tm_code', '');
                    $c_code = $prop->get('c_tm_code', '');

                    self::$customcode[$key] = $code;
                    self::$a_customcode[$key] = $a_code;
                    self::$c_customcode[$key] = $c_code;
                }

                //UL WRAPPER
                if ($prop->get('tm_ul_on', 0)) {

                    $ul_box_shadow = str_replace(',', 'px ', $prop->get('tm_ul_box_shadow', '0,0,0,0')) . 'px';
                    $ul_box_shadow_color = $prop->get('tm_ul_box_shadow_color', 'rgba(0,0,0,0)');
                    $custom_css .= '$' . $key . '_ul_box_shadow:' . $ul_box_shadow . ' ' . $ul_box_shadow_color . ';';

                    $ul_b_color = $prop->get('tm_ul_border_color', 'rgba(0,0,0,0');
                    $custom_css .= '$' . $key . '_ul_border_color:' . $ul_b_color . ';';

                    $ul_b_width = str_replace(',', 'px ', $prop->get('tm_ul_border_width', '0,0,0,0')) . 'px';
                    $ul_b_width_array = explode(',', $prop->get('tm_ul_border_width', '0,0,0,0'));
                    $custom_css .= '$' . $key . '_ul_border_width:' . $ul_b_width . ';';

                    //BORDER RADIUS
                    $ul_b_radius = str_replace(',', 'px ', $prop->get('tm_ul_border_radius', '0,0,0,0')) . 'px';
                    $ul_b_radius_array = explode(',', $prop->get('tm_ul_border_radius', '0,0,0,0'));

                    $ul_b_radius_final = array(
                        ($ul_b_radius_array[0] - $ul_b_width_array[0] >= 0) ? $ul_b_radius_array[0] - $ul_b_width_array[0] : 0,
                        ($ul_b_radius_array[1] - $ul_b_width_array[1] >= 0) ? $ul_b_radius_array[1] - $ul_b_width_array[1] : 0,
                        ($ul_b_radius_array[2] - $ul_b_width_array[2] >= 0) ? $ul_b_radius_array[2] - $ul_b_width_array[2] : 0,
                        ($ul_b_radius_array[3] - $ul_b_width_array[3] >= 0) ? $ul_b_radius_array[3] - $ul_b_width_array[3] : 0,
                    );

                    //BORDER RADIUS child
                    //si es el primer elemento de desktop configured in horiz
                    if ($context == 'desktop' && $desktop_base == 'horizontal' && $key == 'tm_desktop_custom0') {
                        $custom_css .= '$' . $key . '_ul_border_radius:' . $ul_b_radius . ';';
                        $custom_css .= '$' . $key . '_ul_border_radius_first_li:' . $ul_b_radius_final[0] . 'px 0px 0px ' . $ul_b_radius_final[3] . 'px;';
                        $custom_css .= '$' . $key . '_ul_border_radius_last_li: 0px ' . $ul_b_radius_final[1] . 'px ' . $ul_b_radius_final[2] . 'px 0px;';

                    } else {
                        $custom_css .= '$' . $key . '_ul_border_radius:' . $ul_b_radius . ';';
                        $custom_css .= '$' . $key . '_ul_border_radius_first_li:' . $ul_b_radius_final[0] . 'px ' . $ul_b_radius_final[1] . 'px 0px 0px;';
                        $custom_css .= '$' . $key . '_ul_border_radius_last_li: 0px 0px ' . $ul_b_radius_final[2] . 'px ' . $ul_b_radius_final[3] . 'px;';
                    }

                }

            }
        }

        return $custom_css;

    }

    private static function getElements($custom, $params)
    {

        $atribs_obj = $custom;

        $custom_css = '';
        $lvl = 0;
        foreach ($atribs_obj as $key => $prop) {

            //CSS ELEMENT
            if (!is_object($prop)) {
                $prop = json_decode($prop);
            }
            $prop = new Registry($prop->tmcustom);

            //skip level0
            //why we skip level 0 ?
            //if($key == 'tm_desktop_custom0') continue;
            //if($key == 'tm_mobile_custom0') continue;

            $custom_css .= '

			ul.tmlevel-' . $lvl . ' {';

            //ul border radius
            if ($prop->get('tm_ul_on', 0)) {
                $custom_css .= 'border-radius: $' . ($key) . '_ul_border_radius;';
                $custom_css .= 'border-style: solid;';
                $custom_css .= 'border-width: $' . ($key) . '_ul_border_width;';
                $custom_css .= 'border-color: $' . ($key) . '_ul_border_color;';
                $custom_css .= 'box-shadow: $' . ($key) . '_ul_box_shadow;';
            }

            //code
            if ($prop->get('tm_code_on', 0)) {
                $custom_css .= self::$customcode[$key];
            }

            //
            // LI
            //
            $custom_css .= '
				>li{';

            //border radius
            if ($prop->get('tm_ul_on', 0)) {
                $custom_css .= '&:first-child{
							>div.dxgroup{
								border-radius: $' . $key . '_ul_border_radius_first_li;
							}
						}
						&:last-child{
							border-bottom: none;
							>div.dxgroup{
								border-radius: $' . $key . '_ul_border_radius_last_li;
							}
						}';
            }

            //
            // DX GROUP
            //
            $custom_css .= '
					>div.dxgroup{
						';

            //border radius
            if ($prop->get('tm_border_radius_on', 0)) {
                $custom_css .= 'border-radius: $' . ($key) . '_border_radius;';
            }

            if ($prop->get('tm_background_on', 0)) {
                $custom_css .= '
							background: $' . $key . '_bg;
							';
            }
            if ($prop->get('tm_border_width_on', 0)) {
                $custom_css .= '
							border-width: $' . ($key) . '_border_width;
							';
            }

            if ($prop->get('tm_border_color_on', 0)) {
                $custom_css .= '
							border-color: $' . ($key) . '_border_color;
							';
            }

            //
            // A
            //
            $custom_css .= '
						a{';

            if ($prop->get('tm_color_on', 0)) {
                $custom_css .= '
								color: $' . ($key) . '_color; ';
            }
            if ($prop->get('tm_size_on', 0)) {
                $custom_css .= '
								font-size: $' . ($key) . '_size; ';
                $custom_css .= '
								line-height: $' . ($key) . '_size; ';
            }

            if ($prop->get('tm_padding_on', 0)) {
                $custom_css .= '
								padding: $' . ($key) . '_padding; ';
            }

            $custom_css .= '
						}';
            $custom_css .= '
					}';

            ////////////////////////////////////////////////////////////
            // LI hover (active)
            //
            $custom_css .= '
					&:hover{';

            //code
            if ($prop->get('tm_code_on', 0)) {
                $custom_css .= self::$a_customcode[$key];
            }

            //
            // dx group
            //
            $custom_css .= '
						>div.dxgroup{
							';

            if ($prop->get('tm_background_on', 0)) {
                $custom_css .= '
							background: $' . $key . '_a_bg;
							';
            }
            if ($prop->get('tm_border_width_on', 0)) {
                $custom_css .= '
							border-width: $' . ($key) . '_a_border_width;
							';
            }

            if ($prop->get('tm_border_color_on', 0)) {
                $custom_css .= '
							border-color: $' . ($key) . '_a_border_color;
							';
            }

            //
            // A
            //
            $custom_css .= '
							a{';

            if ($prop->get('tm_color_on', 0)) {
                $custom_css .= '
								color: $' . ($key) . '_a_color; ';
            }
            if ($prop->get('tm_size_on', 0)) {
                $custom_css .= '
								font-size: $' . ($key) . '_a_size; ';
                $custom_css .= '
								line-height: $' . ($key) . '_a_size; ';
            }
            if ($prop->get('tm_padding_on', 0)) {
                $custom_css .= '
								padding: $' . ($key) . '_a_padding; ';
            }

            $custom_css .= '
							}';
            $custom_css .= '
						}';
            $custom_css .= '
					}';

            ////////////////////////////////////////////////////////////
            // LI current
            ////////////////////////////////////////////////////////////
            $custom_css .= '
					&.current,&.tm_open{';

            //code
            if ($prop->get('tm_code_on', 0)) {
                $custom_css .= self::$a_customcode[$key];
            }

            //
            // dx group
            //
            $custom_css .= '
						>div.dxgroup{
							';
            //border radius
            if ($prop->get('tm_border_radius_on', 0)) {
                $custom_css .= 'border-radius: $' . ($key) . '_c_border_radius;';
            }
            if ($prop->get('tm_background_on', 0)) {
                $custom_css .= '
								background: $' . $key . '_c_bg;
								';
            }
            if ($prop->get('tm_border_width_on', 0)) {
                $custom_css .= '
								border-width: $' . ($key) . '_c_border_width;
								';
            }

            if ($prop->get('tm_border_color_on', 0)) {
                $custom_css .= '
								border-color: $' . ($key) . '_c_border_color;
								';
            }

            //
            // A
            //
            $custom_css .= '
							a{';

            if ($prop->get('tm_color_on', 0)) {
                $custom_css .= '
								color: $' . ($key) . '_c_color; ';
            }
            if ($prop->get('tm_size_on', 0)) {
                $custom_css .= '
								font-size: $' . ($key) . '_c_size; ';
                $custom_css .= '
								line-height: $' . ($key) . '_c_size; ';
            }
            if ($prop->get('tm_padding_on', 0)) {
                $custom_css .= '
								padding: $' . ($key) . '_c_padding; ';
            }
            //close a
            $custom_css .= '
							}';
            //close dxgroup
            $custom_css .= '
						}';
            //close current
            $custom_css .= '
					}';

            //close LI
            $custom_css .= '
				}';

            //close UL
            $custom_css .= '
			}';

            $lvl++;

        }

        return $custom_css;
    }
    */
/*
    public static function removeWhiteSpace($text)
    {
        $text = preg_replace('/[\t\n\r\0\x0B]/', '', $text);
        $text = preg_replace('/([\s])\1+/', ' ', $text);
        $text = trim($text);
        return $text;
    }

    public static function getCaret($item, $params, $unique = '')
    {

        $tm_mobile_caret = new Registry;
        $tm_mobile_caret->loadString(json_encode($params->get('tm_mobile_caret')));

        $tm_desktop_caret = new Registry;
        $tm_desktop_caret->loadString(json_encode($params->get('tm_desktop_caret')));

        self::$caret_desktop_on = $tm_desktop_caret->get('tm_caret_open', 'fa-arrow-right');
        self::$caret_mobile_on = $tm_mobile_caret->get('tm_caret_open', 'fa-arrow-right');

        $tm_caret_mobile_style = $tm_mobile_caret->get('tm_caret_style', 'fas');
        $tm_caret_mobile_open = $tm_mobile_caret->get('tm_caret_open', '0');
        $tm_alias_mobile_caret = $tm_mobile_caret->get('tm_alias_caret', '0');

        $tm_caret_desktop_style = $tm_desktop_caret->get('tm_caret_style', 'fas');
        $tm_caret_desktop_open = $tm_desktop_caret->get('tm_caret_open', '0');
        $tm_alias_desktop_caret = $tm_desktop_caret->get('tm_alias_caret', '0');

        $tm_version = $params->get('tm_version', 4);

        $onclick = ($tm_version == 4) ? '' : 'onClick="tmOpenSubmenu(this)"';

        $rotate_mode_90deg_array = array('fa-plus', 'fa-plus-square', 'fa-plus-circle');

        //$tm_caret_desktop_style = 'fa';

        //ALIAS
        if ($item->type == 'alias') {

            //mobile
            $faclass_mobile = ($tm_alias_mobile_caret == '0' ? '' : $tm_caret_mobile_style . ' ' . $tm_alias_mobile_caret);
            $fa_a_class_mobile = ($tm_alias_mobile_caret == '0' ? ' no-tmflecha_mobile' : ' tmflecha_mobile');

            //check if the caret must rotate 45 or 90deg
            if (in_array($tm_alias_mobile_caret, $rotate_mode_90deg_array)) {
                $rotate_mobile_mode = ' tm_mobile_rotate_45';
            } else {
                $rotate_mobile_mode = ' tm_mobile_rotate_90';
            }

            //desktop
            $faclass_desktop = ($tm_alias_desktop_caret == '0' ? '' : $tm_caret_desktop_style . ' ' . $tm_alias_desktop_caret);
            $fa_a_class_desktop = ($tm_alias_desktop_caret == '0' ? ' no-tmflecha_desktop' : ' tmflecha_desktop');

            //check if the caret must rotate 45 or 90deg
            if (in_array($tm_alias_desktop_caret, $rotate_mode_90deg_array)) {
                $rotate_desktop_mode = ' tm_desktop_rotate_45';
            } else {
                $rotate_desktop_mode = ' tm_desktop_rotate_90';
            }

            $caret = '<a tabindex="0" '.$onclick.' role="button" aria-haspopup="true" aria-expanded="false" aria-label="' . Text::_('MOD_MENU_TOGGLE') . '" aria-controls="tmulid-' . $item->id . '"
				class="dxsubmenu ' . $fa_a_class_mobile . $fa_a_class_desktop . $rotate_mobile_mode . $rotate_desktop_mode . '" >';
            $caret .= ($faclass_mobile) ? '<i class="caret_mobile ' . $faclass_mobile . '"></i>' : '';
            $caret .= ($faclass_desktop) ? '<i class="caret_desktop ' . $faclass_desktop . '"></i>' : '';
            $caret .= '</a>';
        } else {
            //mobile
            $faclass_mobile = ($tm_caret_mobile_open == '0' ? '' : $tm_caret_mobile_style . ' ' . $tm_caret_mobile_open);
            $fa_a_class_mobile = ($tm_caret_mobile_open == '0' ? ' no-tmflecha_mobile' : ' tmflecha_mobile');

            //check if the caret must rotate 45 or 90deg
            if (in_array($tm_caret_mobile_open, $rotate_mode_90deg_array)) {
                $rotate_mobile_mode = ' tm_mobile_rotate_45';
            } else {
                $rotate_mobile_mode = ' tm_mobile_rotate_90';
            }

            //desktop
            $faclass_desktop = ($tm_caret_desktop_open == '0' ? '' : $tm_caret_desktop_style . ' ' . $tm_caret_desktop_open);
            $fa_a_class_desktop = ($tm_caret_desktop_open == '0' ? ' no-tmflecha_desktop' : ' tmflecha_desktop');

            //check if the caret must rotate 45 or 90deg
            if (in_array($tm_caret_desktop_open, $rotate_mode_90deg_array)) {
                $rotate_desktop_mode = ' tm_desktop_rotate_45';
            } else {
                $rotate_desktop_mode = ' tm_desktop_rotate_90';
            }
   
            $caret = '<a tabindex="0" '.$onclick.' role="button" aria-haspopup="true" aria-expanded="false" aria-label="' . Text::_('MOD_MENU_TOGGLE') . '" aria-controls="tmulid-' . $item->id . '"  class="dxsubmenu ' . $fa_a_class_mobile . $fa_a_class_desktop . $rotate_mobile_mode . $rotate_desktop_mode . '" >';
            $caret .= ($faclass_mobile) ? '<i class="caret_mobile ' . $faclass_mobile . '"></i>' : '';
            $caret .= ($faclass_desktop) ? '<i class="caret_desktop ' . $faclass_desktop . '"></i>' : '';
            $caret .= '</a>';
        }

        return $caret;
    }
*/

}
