<?php

/**
 * @package     Jtotal.Site
 * @subpackage  mod_totalmenu
 *
 * @copyright   (C) 2011 - 2023 JTotal, Inc. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 */

defined('_JEXEC') or die;

use Joomla\CMS\Environment\Browser;
use Joomla\CMS\Factory;
use Joomla\Registry\Registry;
use Joomla\CMS\Uri\Uri;
use Joomla\CMS\Helper\ModuleHelper;
use Joomla\CMS\Profiler\Profiler;
use Jtotal\Module\Totalmenu\Site\Helper\TotalmenuCatMode;
use Jtotal\Module\Totalmenu\Site\Helper\TotalmenuDebug;
use Jtotal\Module\Totalmenu\Site\Helper\TotalmenuHelper;
use Jtotal\Module\Totalmenu\Site\Helper\TotalmenuProcessCss;

// include the syndicate functions only once
require_once dirname(__FILE__) . '/helper.php';
require_once __DIR__ . '/src/Helper/TotalmenuDebug.php';
//require_once __DIR__ . '/src/Helper/TotalmenuProcessCss.php';

$document = $app->getDocument();
$wa = $document->getWebAssetManager();
$tm_version = $params->get('tm_version', 4);

$debug = $params->get('visual_menu_debug', 0);
if ($debug) {
    $profiler = new Profiler();
}

///////////////////
//  CHECK FW //
///////////////////
/*
if (is_file(JPATH_PLUGINS . '/system/jtframework/autoload.php')) {
    require_once JPATH_PLUGINS . '/system/jtframework/autoload.php';
} else {
    $app->enqueueMessage(Text::_('JT_FW_NOT_FOUND'), 'error');
    return;
}
*/

//deprecated
$tm_debug = $params->get('tm_debug', '0');


///////////////////
//  PARAMS //
///////////////////
//TODO change unique by $module->id

$unique = htmlspecialchars($params->get('dx_uniquename').$module->id);
$modulename = $unique;




$tm_desktop_behavior_click = $params->get('tm_desktop_click_hover', '0');
$tm_force_reloadcss = $params->get('tm_force_reloadcss', '0') == '1' ? '?forcereloadcss=' . rand() : '';
$tm_force_reloadcssv5 = $params->get('tm_force_reloadcss', '0') == '1' ? md5(rand()) : 'auto';


//js PARAMS //
$tm_js_params = array();


//CACHE
//CACHE
//CACHE




$cacheParams = new stdClass;

$cacheParams->class = 'Jtotal\Module\Totalmenu\Site\Helper\TotalmenuHelper';

$cacheMode = $params->get('cachemode', 'safeuri');
$jinput = $app->input;
$cacheid = md5('totalmenu_' . serialize([$module->module, $jinput->get('id', ''), $jinput->get('itemid', ''), Factory::getLanguage()->getTag()]));

switch ($cacheMode) {
    case 'id':
        $cacheParams->modeparams = $cacheid;
        break;
    case 'safeuri':
        $cacheParams->modeparams = ['id' => 'int', 'Itemid' => 'int'];
        break;
    case 'static':
        $cacheParams->modeparams = $module->id;
        break;
    case 'itemid':
        $cacheParams->modeparams = $module->id;
        break;
    default:
        $cacheParams->modeparams = ['id' => 'int', 'Itemid' => 'int'];
        break;
}

$cacheParams->cachemode =  $cacheMode;

/**
 * MENU MODE: MENU, MENUES, CATEGORY
 */
$tmmode = $params->get('tmmode', 'menu');



$lists = array();
$lists_dev = array();

if ($tmmode == 'category') {
    $cacheParams->method = 'getCatList';
    $cacheParams->methodparams = $params;
    $lists[]    = TotalmenuCatMode::getCatList($params);
} elseif ($tmmode == 'menues') {
    //bucle for menues
    foreach ($params->get('menutype_multiple_subform', array()) as $key => $menues) {
        $cacheParams->method = 'getList';
        $cacheParams->methodparams = $params;
        $cacheid = md5('totalmenu_' . serialize([$module->module, $menues->menutype_multiple, $jinput->get('id', ''), $jinput->get('itemid', ''), Factory::getLanguage()->getTag()]));
        $cacheParams->modeparams = $cacheid;
        $params->set('items_menu', $menues->menutype_multiple);
        //$lists[]    = TotalmenuHelper::getList($params);
        $lists[] = ModuleHelper::moduleCache($module, $params, $cacheParams);
    }
} else {
    $cacheParams->method = 'getList';
    $cacheParams->methodparams = $params;
    $params->set('items_menu', $params->get('menutype'));
    
    $lists[] = ModuleHelper::moduleCache($module, $params, $cacheParams);
}




//check if there is no elements
if (isset($lists[0])) {
    if (count($lists[0]) == 0) {
        return;
    }
}

$base       = TotalmenuHelper::getBase($params);
$active     = TotalmenuHelper::getActive($params);
$active_id  = $active->id;
$path       = $base->tree;

$menu       = $app->getMenu();
$menu_parent= $menu->getItem($path[0]);

$showAll    = $params->get('showAllChildren', 1);

$dx_fixedtop = $params->get('dx_fixedtop', 0);
$fixedtop = $dx_fixedtop ? ' navbar-fixed-top' : '';

$dx_brand_on = $params->get('dx_brand_on', '0');


$dx_brand = $params->get('dx_brand', '');
$dx_brand_fallback = $params->get('dx_brand_fallback', '');
$dx_brand_alt = $params->get('dx_brand_alt', 'logo');
$dx_brand_class = $params->get('dx_brand_class', '');


//BRAND BS
$dx_brand_lg = $params->get('dx_brand_bs_lg', '4');
$dx_brand_md = $params->get('dx_brand_bs_md', '4');
$dx_brand_sm = $params->get('dx_brand_bs_sm', '3');
$dx_brand_xs = $params->get('dx_brand_bs_xs', '12');

$dx_brand_link = $params->get('dx_brand_link', 'index.php');

$detection = $params->get('tm_mobile_detection', 'media');

//IMAGE BRAND
//TODO HTMLHelper::cleanImageURL($dx_brand);
//remove #joomlaImage://
$class_sfx = ($params->get('class_sfx', '') == '') ? '' : htmlspecialchars($params->get('class_sfx', ''));
$subwidth = $params->get('tm_subwidth_100', '1');


//CARET
$tm_mobile_caret = new Registry;
$tm_mobile_caret->loadString(json_encode($params->get('tm_mobile_caret')));

$tm_desktop_caret = new Registry;
$tm_desktop_caret->loadString(json_encode($params->get('tm_desktop_caret')));


$tm_position_mobile = ($params->get('tm_position_mobile', 1) ? 'tm_reverse' : '');
$tm_collapsed = ($params->get('tm_closed_mobile', 0) ? 'tm_collapsed tm_collapsed_done' : '');

$tm_show_collapsed = ($params->get('tm_collapse_button', 0) ? 'tm_show_button' : 'tm_hide_button');

$tm_animations = $params->get('tm_animations', 'tm_animation_basic');
$tm_zindex = $params->get('tm_zindex', '1');


$tm_justify = $params->get('tm_justify_desktop', 'tm_flex_start');
$tm_justify_desktop_wrapper = $params->get('tm_justify_desktop_wrapper', 'tm_flex_start');

$tm_wrapper_max_width = $params->get('tm_wrapper_max_width', '600px');


$dx_brand_max_w = $params->get('dx_brand_max_w', '250');

$tm_desktop_image_pos = $params->get('tm_desktop_image_pos', 'horizontal');

$tm_desktop_wrap = $params->get('tm_desktop_wrap', 'wrap');

$tm_parent_element = $params->get('tm_parent_element', 'alias');


$tm_mobile_media_w  = $params->get('tm_mobile_media_w', 768);


/**************/
/*JS variables*/
/**************/

//Other options
$tm_customcolor = $params->get('tm_customcolor', '0');

//$document->addScriptDeclaration("var php_tm_customcolor = '{$tm_customcolor}';");

$tm_sticky = $params->get('tm_sticky', '0');
$tm_js_params['tm_sticky'] = $params->get('tm_sticky', '0');


$tm_juri = URI::base();
$tm_js_params['tm_juri'] = URI::base();

//start opened mobile
$tm_active_opened_mobile = $params->get('tm_active_opened_mobile', '1');

$tm_js_params['tm_active_opened_mobile'] = $params->get('tm_active_opened_mobile', '1');


if (count($lists)) {
    //load_fa
    if ($params->get('tm_load_fa', '1')) {
        $wa->useStyle('fontawesome');
    }


    //SHOW PARENT
    $tm_desktop_show_parent = $params->get('tm_desktop_show_parent', '0');


    ////////////
    // STYCKY //
    ////////////

    $tm_sticky_mobile = $params->get('tm_sticky_mobile', '0');
    $tm_sticky_mobile_top = $params->get('tm_sticky_mobile_top', '0');

    $tm_sticky_desktop = $params->get('tm_sticky_desktop', '0');
    $tm_sticky_desktop_top = $params->get('tm_sticky_desktop_top', '0');



    /////////////////////
    // START OPENED // //
    /////////////////////

    $tm_startopened_desktop = 'tm_startopened_desktop_' . $params->get('tm_startopened_desktop', '0');
    $tm_startopened_mobile = 'tm_startopened_mobile_' . $params->get('tm_startopened_mobile', '0');


    /////////////////////
    // HOVER CLICK // //
    /////////////////////

    $hover_click = 'tm_desktop_behavior_' . $params->get('tm_desktop_click_hover', 'click');



    //duplicate in xml to set value in database
    $tm_production = $params->get('tm_production', '0');


    $tm_mobile_base = $params->get('tm_mobile_base', 'vertical');
    $tm_mobile_fade = $params->get('tm_mobile_fadein', 300);
    $tm_mobile_style = $params->get('tm_mobile_style', 'custom');

    $tm_portal = $params->get('tm_mobile_media_portal', '');
    $tm_portal_pos = $params->get('tm_mobile_portal_pos', 'start');

    $tm_desktop_base = $params->get('tm_desktop_base', 'horizontal');
    $tm_desktop_fade = $params->get('tm_desktop_fadein', 300);
    $tm_desktop_style = $params->get('tm_desktop_style', 'custom');


    $tm_js_params['moduleid'] = $module->id;
    $tm_js_params['tm_production'] = $tm_production;

    $tm_js_params['tm_portal'] = $tm_portal;
    $tm_js_params['tm_portal_pos'] = $tm_portal_pos;


    //CLASSES
    $tm_classes = 'totalmenu_wrapper 
        tm_behavior_mobile_' . $tm_mobile_base . ' 
        tm_behavior_desktop_' . $tm_desktop_base . ' 
        subwidth_' . $subwidth . ' ' .
        $tm_animations . ' 
        tm_sticky_mobile_' . $tm_sticky_mobile . ' 
        tm_sticky_desktop_' . $tm_sticky_desktop . ' ' .
        $tm_startopened_desktop . ' ' .
        $tm_startopened_mobile . ' ' .
        $tm_justify_desktop_wrapper . ' ' .
        $hover_click . ' 
        image_pos_' . $tm_desktop_image_pos . ' 
        tm_' . $tm_desktop_wrap . ' 
        tm_customcolor_' . $tm_customcolor . ' 
        tm_loading';

    $tm_classes = TotalmenuHelper::removeWhiteSpace($tm_classes);

    TotalmenuDebug::addDebug("Classes", $tm_classes, 'id-card-alt');

    $params_to_js = json_encode(array('params' => $tm_js_params));
 
    /**************/
    /* VERSION    */
    /**************/
    if ($tm_version == 4) {
        include 'src/Helper/loadCssV4.php';
    } else {
        //version 5
        //version 5
        //version 5
        //version 5
        //version 5

        // Comprobar si el usuario está en un móvil
        if (Browser::getInstance()->isMobile()) {
            // El usuario está en un móvil
        } else {
            // El usuario no está en un móvil
        }


        $tm_jmobile = 0;
 

        ///V4 DEPENDANT WILL DEP IN FUTURE
        //check file exist
        $desktop_file = 'cache/mod_totalmenu/' . $unique . '_desktop_' . $tm_desktop_style . '.css';
        $css_file_desktop_exists = file_exists($desktop_file);

        //check MOBILE file exist
        $mobile_file = 'cache/mod_totalmenu/' . $unique . '_mobile_' . $tm_mobile_style . '.css';
        $css_file_mobile_exists = file_exists($mobile_file);


        //check MOBILE file exist
        $mix_file = 'cache/mod_totalmenu/' . $unique . '.css';
        $mix_file_exists = file_exists($mix_file);

        if (!$mix_file_exists || !$tm_production) {
            $mix_file = TotalmenuProcessCss::getCSS($params, $module->id);
        }
        $mixCss = $mix_file;

        $tm_loadcss = $params->get('tm_loadcss', '1');

        ///LOAD CSS
        $assetOptions = ['version' => $tm_force_reloadcssv5];
        $assetAttribs = [];

        $tm_lazy_load = $params->get('tm_lazy_load', '0');

        if ($tm_lazy_load == '1') {
            $assetAttribs['media'] = 'print';
            $assetAttribs['rel'] = 'lazy-stylesheet';
            $assetAttribs['onload'] = 'this.media=\'all\'';
            $assetAttribs['crossorigin'] = 'anonymous';
        }

        if ($tm_loadcss == '1') {
            $wa->registerAndUseStyle('totalmenu'.$module->id, $mixCss, $assetOptions, $assetAttribs);
        }


        $tm_mobile_start_open = ($params->get('tm_startopened_mobile', '1')) ? 'true' : 'false';
        $tm_desktop_start_open = ($params->get('tm_startopened_desktop', '1')) ? 'true' : 'false';

        //JS VARIABLES
        $wa->addInlineSCript('
            if(typeof _tm == "undefined") _tm = {};
        
            let tm_'.$unique.' = {}; 
            let desk_dir = "'.$tm_desktop_base.'";
            let deskStartOpen = "'.$tm_desktop_start_open.'";
            let desk_fade = '.$tm_desktop_fade.';

            let mobile_dir = "'.$tm_mobile_base.'";
            let mobileStartOpen = "'.$tm_mobile_start_open.'";
            let mobile_fade = '.$tm_mobile_fade.';

            let mediaw = '.$tm_mobile_media_w.';
            let tmPortalDestiny = "'.$tm_portal.'";

            _tm.tm_'.$unique.' = {desk_dir,deskStartOpen,desk_fade,mobile_dir,mobile_fade,mobileStartOpen,mediaw, tmPortalDestiny};
           
        ');
    }


    /**************/
    /* JS LOADING */
    /**************/
    $wa->getRegistry()->addExtensionRegistryFile('mod_totalmenu');

    if ($tm_version == 5) {
        $wa->useScript('totalmenu5.js');
    } else {
        $wa->useScript('totalmenu4.js');
    }

    require(ModuleHelper::getLayoutPath('mod_totalmenu', $params->get('layout', 'default')));


    echo TotalmenuDebug::showDebug($params);
}
