<?php
/**
 * @package     Totalmenu
 * @subpackage  mod_totalmenu debug
 *
 * @copyright   (C) 2021 jtotal based on Joomla
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 */

namespace Jtotal\Module\Totalmenu\Site\Helper;

use Joomla\CMS\Cache\CacheControllerFactoryInterface;
use Joomla\CMS\Factory;
use Joomla\CMS\Filter\OutputFilter;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Router\Route;
use Joomla\Registry\Registry;
use Joomla\CMS\Component\ComponentHelper;

// phpcs:disable PSR1.Files.SideEffects
\defined('_JEXEC') or die;
// phpcs:enable PSR1.Files.SideEffects

//TODO
abstract class TotalmenuCatMode
{

    public static function getCatList(&$params)
    {
        $app = Factory::getApplication();
        $menu = $app->getMenu();

        // Get active menu item
        $base = TotalmenuHelper::getBase($params);
        $user = Factory::getUser();
        $levels = $user->getAuthorisedViewLevels();
        asort($levels);
        $key = 'totalmenu_cat_to_arts__' . serialize($params->get('dx_uniquename', 'totalmenu'));

        $cache = Factory::getContainer()->get(CacheControllerFactoryInterface::class)
            ->createCacheController('output', ['defaultgroup' => 'mod_totalmenu']);

        // Get an instance of the generic articles model
        //$articles = JModelLegacy::getInstance('Articles', 'ContentModel', array('ignore_request' => true));
        $articles = Factory::getApplication()->bootComponent('com_content')->getMVCFactory()->createModel('Articles', 'Site');

        $appParams = $app->getParams();

        //enable cache

        if ($cache->contains($key)) {
            $articles = $cache->get($key);
        } else {
            $path = $base->tree;
            $start = (int) $params->get('startLevel');
            $end = (int) $params->get('endLevel');
            $showAll = $params->get('showAllChildren', 1);
            //$items          = $menu->getItems('menutype', $params->get('menutype'));
            $hidden_parents = array();
            $lastitem = 0;

            //Set application parameters in model

            $articles->setState('params', $appParams);

            // Access filter
            $access = !ComponentHelper::getParams('com_content')->get('show_noauth');
            $authorised = $user->getAuthorisedViewLevels(Factory::getUser()->get('id'));
            $articles->setState('filter.access', $access);

            $articles->setState('filter.published', 1);
            $articles->setState('filter.category_id.include', (bool) true);

            // Featuring
            $featured = $params->get('tmcategory_show_article_featured');

            //only
            if ($featured == '2') {
                $articles->setState('filter.featured', 'only');
            } elseif ($featured == '1') {
                $articles->setState('filter.featured', 'show');
            } else {
                $articles->setState('filter.featured', 'hide');
            }

            // Ordering
            $articles->setState('list.ordering', 'publish_up');
            $articles->setState('list.direction', $params->get('tmcategory_orderby_asc', 'ASC'));

            //////////////////
            //CATs
            //////////////////
            $catids = $params->get('tmcategory_category', []);

            //$categories = JModelLegacy::getInstance('Categories', 'ContentModel', array('ignore_request' => true));
            $categories = Factory::getApplication()->bootComponent('com_content')->getMVCFactory()->createModel('Categories', 'Site');

            $categories->setState('params', $appParams);

            //$categories->setState('filter.get_children', $levels);
            $categories->setState('filter.published', 1);
            $categories->setState('filter.access', $access);

            $additional_catids = array();
            $cat_tree = array();

            //echo "categorias hijo: ";
            //tmcategory_children
            if ($params->get('tmcategory_children', 0)) {
                foreach ($catids as $catid) {

                    $categories->setState('filter.parentId', $catid);
                    $recursive = true;
                    $items = $categories->getItems($recursive);

                    if ($items) {
                        foreach ($items as $category) {
                            $condition = (($category->level - $categories->getParent()->level) <= $levels);
                            if ($condition) {
                                $additional_catids[] = (string) $category->id;
                                //echo $category->id."  |   ";
                            }
                        }
                    }
                }
            }

            $catids = array_unique(array_merge($catids, $additional_catids));
            $articles->setState('filter.category_id', $catids);

            //$cache->store($articles, $key);
        }

        //var_dump($articles->getItems());
        $dateTree = self::articlesToItemTreev2($articles->getItems(), $params);

        $items = self::itemsToMenuV2($dateTree, $params);

        return $items;
    }

    public static function getScheme($params = "")
    {

        $esqueleto = array(
            'id' => '',
            'menutype' => 'totalmenu_dynamic',
            'title' => '',
            'alias' => '',
            'route' => '',
            'link' => '#',
            'type' => 'alias',
            'level' => '1',
            'language' => '*',
            'browserNav' => 'nav',
            'access' => '1',
            'parent_id' => '0',
            'home' => '0',
            'img' => '0',
            'tree' => array(),

            'anchor_css' => '',
            'anchor_title' => '',
            'anchor_rel' => '',

            'menu_icon' => '',
            'menu_image' => '',
            'browserNav' => '',
            'flink' => '#',

            'params' => $params,
        );
        return $esqueleto;
    }

    public static function articlesToItemTreev2($items, $params)
    {

        $esqueleto = self::getScheme($params);

        $year_on = in_array('year', $params->get('tmcategory_date_parent', array()));
        $month_on = in_array('month', $params->get('tmcategory_date_parent', array()));
        $day_on = in_array('day', $params->get('tmcategory_date_parent', array()));
        $cat_on = in_array('cat', $params->get('tmcategory_date_parent', array()));
        $art_on = in_array('art', $params->get('tmcategory_date_parent', array()));

        $treeControl = array();
        $f_menu = array();
        $cuenta_articles = 0;

        foreach ($items as $key => $item) {
            $level = 1;

            $timestamp = strtotime($item->publish_up);
            $year = date("Y", $timestamp);
            $month = date("M", $timestamp);
            $day = date("d", $timestamp);

            $current_parent = 0;

            $current_tree = array();

            //YEARS
            if ($year_on) {
                $common = $esqueleto;
                $common['id'] = $year;
                $common['title'] = $year;
                $common['route'] = $year;
                $common['level'] = $level;
                $common['alias'] = OutputFilter::stringURLSafe($year);
                $common['parent_id'] = 0;
                $common['tree'][] = $common['id'];

                $current_tree = $common['tree'];
                $current_parent = $common['id'];
                //add to control
                if (!isset($treeControl[$year])) {
                    $treeControl[$year] = [];
                    $f_menu[] = $common;
                }

                $level++;
            }

            //MONTH
            if ($month_on) {
                $common = $esqueleto;
                $common['id'] = $month;
                $common['title'] = '' . Text::sprintf(HTMLHelper::_('date', $month, 'F'));
                $common['route'] = $month;
                $common['level'] = $level;
                $common['alias'] = OutputFilter::stringURLSafe($month);
                $common['parent_id'] = $current_parent;

                $current_parent = $common['id'];

                $current_tree[] = $common['id'];
                $common['tree'] = $current_tree;

                //add to control
                if (!isset($treeControl[$year][$month])) {
                    $treeControl[$year][$month] = [];
                    $f_menu[] = $common;
                }

                $level++;
            }

            //DAY
            if ($day_on) {
                $common = $esqueleto;
                $common['id'] = $day;
                $common['title'] = 'day_' . $day;
                $common['route'] = $day;
                $common['level'] = $level;
                $common['alias'] = OutputFilter::stringURLSafe($day);
                $common['parent_id'] = $current_parent;

                $current_parent = $common['id'];

                $current_tree[] = $common['id'];
                $common['tree'] = $current_tree;

                if (!isset($treeControl[$year][$month][$day])) {
                    $treeControl[$year][$month][$day] = [];
                    $f_menu[] = $common;
                }

                $level++;
            }

            //CAT
            if ($cat_on) {
                $common = $esqueleto;
                $common['id'] = $item->catid;
                if ($params->get('tm_debug', 0)) {
                    $common['title'] = 'cat_' . $item->category_title;
                } else {
                    $common['title'] = $item->category_title;
                }
                $common['route'] = $item->category_route;
                $common['level'] = $level;
                $common['alias'] = $item->category_alias;
                $common['parent_id'] = $current_parent;
                $common['tree'][] = $common['id'];
                $common['type'] = 'category';
                $common['catid'] = $item->catid;

                $current_tree[] = $common['id'];
                $common['tree'] = $current_tree;
                //category LINK
                if ($params->get('tmcategory_category_link', 0) == '0') {
                    $common['type'] = 'alias';
                }

                if (!isset($treeControl[$year][$month][$day][$item->catid])) {
                    $treeControl[$year][$month][$day][$item->catid] = [];
                    $f_menu[] = $common;
                    $current_parent = $common['id'];
                }

                $level++;
            }

            //ART
            if ($art_on) {
                $common = $esqueleto;
                $common['id'] = $item->id;
                if ($params->get('tm_debug', 0)) {
                    $common['title'] = 'art_' . $item->title;
                } else {
                    $common['title'] = $item->title;
                }
                $common['route'] = $item->alias;
                $common['level'] = $level;
                $common['alias'] = $item->alias;
                $common['parent_id'] = $current_parent;
                $common['tree'][] = $common['id'];
                $common['type'] = 'article';
                $common['catid'] = $item->catid;
                $common['category_alias'] = $item->category_alias;

                $current_tree[] = $common['id'];
                $common['tree'] = $current_tree;

                if (!isset($treeControl[$year][$month][$day][$item->catid][$item->id])) {
                    $treeControl[$year][$month][$day][$item->catid][$item->id] = [];
                    $f_menu[] = $common;
                    $current_parent = $common['id'];
                }

                $level++;
            }
        }

        $final_menu_to_items = array();
        $final_control_duplicates = array();
        foreach ($f_menu as $key => $value) {
            if (!in_array($value['id'] . '_' . $value['parent_id'], $final_control_duplicates)) {
                $final_control_duplicates[] = $value['id'] . '_' . $value['parent_id'];
                $final_menu_to_items[] = (object) $value;
            }
        }
        return $final_menu_to_items;
    }

    public static function itemsToMenuV2($dateTree, $params)
    {

        $app = Factory::getApplication();
        $menu = $app->getMenu();

        $jinput = Factory::getApplication()->input;
        $id = $jinput->getInt('id', '');
        $parent_id = $jinput->getInt('parent_id', '');
        $itemid = $jinput->get('Itemid', '');
        $catid = $jinput->getInt('catid', '');
        $view = $jinput->get('view', '');

        // Get active menu item
        $base = TotalmenuHelper::getBase($params);
        $user = Factory::getUser();
        $levels = $user->getAuthorisedViewLevels();
        asort($levels);
        $key = 'totalmenu_art_to_items' . $params . implode(',', $levels) . '.' . $base->id;

        $cache = Factory::getContainer()->get(CacheControllerFactoryInterface::class)
                ->createCacheController('output', ['defaultgroup' => 'mod_totalmenu']);

        //if(false)
        if ($cache->contains($key)) {
            $items = $cache->get($key);
        } else {
            $path = $base->tree;
            $start = (int) $params->get('startLevel'); //1
            $end = (int) $params->get('endLevel'); //0
            $showAll = $params->get('showAllChildren'); //1
            $items = $dateTree;
            $hidden_parents = array();
            $lastitem = 0;

            if ($items) {
                foreach ($items as $i => $item) {
                    $item->parent = false;
                    $itemParams = new Registry($item->params);

                    if (isset($items[$lastitem]) && $items[$lastitem]->id == $item->parent_id && $itemParams->get('menu_show', 1) == 1) {
                        $items[$lastitem]->parent = true;
                    }

                    if ($params->get('tmmode', 'menu') == 'menu') {
                        if (($start && $start > $item->level)
                            || ($end && $item->level > $end)
                            || (!$showAll && $item->level > 1 && !in_array($item->parent_id, $path))
                            || ($start > 1 && !in_array($item->tree[$start - 2], $path))) {
                            unset($items[$i]);
                            continue;
                        }
                    }

                    // Exclude item with menu item option set to exclude from menu modules
                    if (($itemParams->get('menu_show', 1) == 0) || in_array($item->parent_id, $hidden_parents)) {
                        $hidden_parents[] = $item->id;
                        unset($items[$i]);
                        continue;
                    }

                    $item->deeper = false;
                    $item->shallower = false;
                    $item->level_diff = 0;

                    if (isset($items[$lastitem])) {
                        $items[$lastitem]->deeper = ($item->level > $items[$lastitem]->level);
                        $items[$lastitem]->shallower = ($item->level < $items[$lastitem]->level);
                        $items[$lastitem]->level_diff = ($items[$lastitem]->level - $item->level);
                    }

                    $lastitem = $i;
                    $item->active = false;
                    $item->flink = $item->link;

                    if ($id == $item->id) {
                        $item->active = true;
                    }

                    // Reverted back for CMS version 2.5.6
                    switch ($item->type) {
                        case 'separator':
                            break;

                        case 'heading':
                            // No further action needed.
                            break;

                        case 'url':
                            if ((strpos($item->link, 'index.php?') === 0) && (strpos($item->link, 'Itemid=') === false)) {
                                // If this is an internal Joomla link, ensure the Itemid is set.
                                $item->flink = $item->link . '&Itemid=' . (int) $item->id;
                            }
                            break;

                        case 'alias':
                            $item->flink = 'index.php?Itemid=' . $itemParams->get('aliasoptions');
                            break;

                        case 'category':
                            $item->active = false;
                            if ($id == $item->id && $view == 'category' && $item->parent_id == $parent_id) {
                                $item->active = true;
                            }

                            //http://localhost/online/j3_s2sv4_dev/index.php?option=com_content&view=category&id=22&Itemid=1982
                            //$item->flink = 'index.php?option=com_content&view=category&layout=blog&parent_id='.$item->parent_id.'&id='.(int)$item->catid;
                            $item->flink = Route::_('index.php?option=com_content&view=category&parent_id=' . $item->parent_id . '&id=' . (int) $item->id);

                            break;

                        case 'article':
                            $item->active = false;
                            if ($catid == $item->catid && $id == $item->id && $view == 'article') {
                                $item->active = true;
                                /*                 var_dump($item->title);
                            var_dump($item->id.' = '.$id); */
                            }

                            $link = Route::_("index.php?option=com_content&view=article&id=" . (int) $item->id);
                            //$item->flink = JRoute::_("index.php?option=com_content&view=article&parent_id=".(int)$item->parent_id."&id=".(int)$item->id);
                            $item->flink = Route::_('index.php?option=com_content&view=article&id=' . (int) $item->id . ':' . (string) $item->alias . '&catid=' . (int) $item->catid . ':' . (string) $item->category_alias);

                            break;

                        default:
                            $item->flink = 'index.php?Itemid=' . $item->id;
                            break;
                    }

                    if ((strpos($item->flink, 'index.php?') !== false) && strcasecmp(substr($item->flink, 0, 4), 'http')) {
                        $item->flink = Route::_($item->flink, true, $item->params->get('secure'));
                    } else {
                        $item->flink = Route::_($item->flink);
                    }

                    // We prevent the double encoding because for some reason the $item is shared for menu modules and we get double encoding
                    // when the cause of that is found the argument should be removed
                    $item->title = htmlspecialchars($item->title, ENT_COMPAT, 'UTF-8', false);
                    $item->anchor_css = htmlspecialchars($itemParams->get('menu-anchor_css', ''), ENT_COMPAT, 'UTF-8', false);
                    $item->anchor_title = htmlspecialchars($itemParams->get('menu-anchor_title', ''), ENT_COMPAT, 'UTF-8', false);
                    $item->anchor_rel = htmlspecialchars($itemParams->get('menu-anchor_rel', ''), ENT_COMPAT, 'UTF-8', false);
                    $item->menu_image = $item->params->get('menu_image', '') ?
                    htmlspecialchars($item->params->get('menu_image', ''), ENT_COMPAT, 'UTF-8', false) : '';
                    $item->menu_image_css = htmlspecialchars($itemParams->get('menu_image_css', ''), ENT_COMPAT, 'UTF-8', false);
                }

                if (isset($items[$lastitem])) {
                    $items[$lastitem]->deeper = (($start ?: 1) > $items[$lastitem]->level);
                    $items[$lastitem]->shallower = (($start ?: 1) < $items[$lastitem]->level);
                    $items[$lastitem]->level_diff = ($items[$lastitem]->level - ($start ?: 1));
                }
            }

            $cache->store($items, $key);
        }

        return $items;
    }
}
