<?php
/**
 * @package     Totalmenu
 * @subpackage  mod_totalmenu debug
 *
 * @copyright   (C) 2021 jtotal based on Joomla
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 */

namespace Jtotal\Module\Totalmenu\Site\Helper;

use Joomla\CMS\Factory;
use Joomla\CMS\Profiler\Profiler;

\defined('_JEXEC') or die;

abstract class TotalmenuDebug
{

    public static $debug_msg = array();

    public static function addDebug($func = '', $msg = '', $icon = '')
    {
        $cuenta = count(self::$debug_msg);
        self::$debug_msg[$cuenta]['func'] = $func;
        self::$debug_msg[$cuenta]['msg'] = $msg;
        self::$debug_msg[$cuenta]['icon'] = $icon;
    }

    public static function getDebug()
    {
        return self::$debug_msg;
    }

    public static function showDebug($params)
    {

        $tm_debug = $params->get('tm_debug', '0');

        if (!$tm_debug) {
            return;
        }

        $html = '';
        $html_profiler = '';
        $profiler = new Profiler();

        $msgs = self::getDebug();

        if ($params->get('cache', 0)) {
            $app = Factory::getApplication();
            $config = Factory::getConfig();
            $global_cache = $config->get('caching');

            if ($global_cache) {
                $html_profiler .= $profiler->mark(' with caching');
            } else {
                $html_profiler .= $profiler->mark(' with caching in module, but disabled in config');
            }
        } else {
            $html_profiler .= $profiler->mark(' no caching');
        }

        $html .= '<div class="table-responsive"><table class="table table-striped table-condensed table-bordered "><thead><tr class="alert alert-success">
        <th class="text-center"><i class="fas fa-bug"></i></th>
        <th colspan="2" class="text-center">' . $html_profiler . '</th>
        </tr></thead>';
        foreach ($msgs as $key => $value) {
            $icon = ($value['icon'] == '') ? '' : '<i class="fas fa-' . $value['icon'] . '"></i>';

            $html .= '<tr><td>' . $icon . '</td><td>' . $value['func'] . '</td><td>' . $value['msg'] . '</td></tr>';
        }

        $html .= '</table></div>';

        return $html;
    }
}
