<?php
/**
 * @package     Totalmenu
 * @subpackage  mod_totalmenu debug
 *
 * @copyright   (C) 2021 jtotal based on Joomla
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 */

namespace Jtotal\Module\Totalmenu\Site\Helper;

use Joomla\CMS\Cache\CacheControllerFactoryInterface;
use Joomla\CMS\Environment\Browser;
use Joomla\CMS\Factory;
use Joomla\CMS\Language\Multilanguage;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Menu\MenuItem;
use Joomla\CMS\Router\Route;
use Joomla\Registry\Registry;

// phpcs:disable PSR1.Files.SideEffects
\defined('_JEXEC') or die;
// phpcs:enable PSR1.Files.SideEffects

//TODO
abstract class TotalmenuHelper
{

    private static $customcode = array();
    private static $a_customcode = array();
    private static $c_customcode = array();

    //caret
    public static $caret_desktop_on;
    public static $caret_mobile_on;
    public static $caret_current_on;

    /**
     * Get a list of the menu items.
     *
     * @param   \Joomla\Registry\Registry  &$params  The module options.
     *
     * @return  array
     *
     * @since   1.5
     */
    public static function getList(&$params)
    {

        $tmmode = $params->get('tmmode', 'menu');

        $app = Factory::getApplication();
        $menu = $app->getMenu();

        // Get active menu item
        $base = self::getBase($params);
        $levels = Factory::getUser()->getAuthorisedViewLevels();
        asort($levels);

        // Compose cache key
        $cacheKey = 'menu_items' . $params . implode(',', $levels) . '.' . $base->id;

        $cache = Factory::getContainer()->get(CacheControllerFactoryInterface::class)
                ->createCacheController('output', ['defaultgroup' => 'mod_menu']);

        if ($cache->contains($cacheKey)) {
            $items = $cache->get($cacheKey);
        } else {
            $path = $base->tree;


            //GLOBAL
            $start = (int) $params->get('startLevel', 1);
            $end = (int) $params->get('endLevel', 0);

            //START AND END LEVELS FOR MOBILE AND DESKTOP
            if (Browser::getInstance()->isMobile()) {
                if($params->get('startLevelMobile', '-1') != '-1'){
                    $start = (int) $params->get('startLevelMobile', 1);
                }
                if($params->get('endLevelMobile', '-1') != '-1'){
                    $end = (int) $params->get('endLevelMobile', 1);
                }
            }else{
                if($params->get('startLevelDesktop', '-1') != '-1'){
                    $start = (int) $params->get('startLevelDesktop', 1);
                }
                if($params->get('endLevelDesktop', '-1') != '-1'){
                    $end = (int) $params->get('endLevelDesktop', 1);
                }
            }


            $showAll = $params->get('showAllChildren', 1);
            //$items          = $menu->getItems('menutype', $params->get('menutype'));
            $hidden_parents = array();
            $lastitem = 0;

            $superItems = array();

            //SUPERPARENT MENU
            $superMenuItem = new MenuItem();
            $superMenuItem->id = 0;
            $superMenuItem->title = $params->get('items_menu', '');
            $superMenuItem->level = '1';
            $superMenuItem->type = 'superparent';

            //MENU
            if ($tmmode == 'menu') {
                $items = array();
                $items = $menu->getItems('menutype', $params->get('menutype'));
            }

            //MENUES
            if ($tmmode == 'menues') {
                $items[] = $superMenuItem;

                foreach ($menu->getItems('menutype', $params->get('items_menu', '')) as $key => $menu) {
                    //if highest level in tree is not supermenu id...
                    if ($menu->tree[0] != $superMenuItem->id) {
                        array_unshift($menu->tree, $superMenuItem->id);
                    }

                    $menu->level = (string) ((int) $menu->level + 1);
                    //$menu->parent = (string) ((int) $menu->level + 1);

                    $items[] = $menu;
                }
            }

            if ($items) {
                $inputVars = $app->getInput()->getArray();
 
                foreach ($items as $i => $item) {
                    //SUPERPARENT
                    $item->parent = false;
                    $itemParams = $item->getParams();

                    //IS PARENT?
                    if (isset($items[$lastitem]) &&
                        $items[$lastitem]->id == $item->parent_id &&
                        $itemParams->get('menu_show', 1) == 1
                    ) {
                        $items[$lastitem]->parent = true;
                    }

                    if (($start && $start > $item->level)
                        || ($end && $item->level > $end)
                        || (!$showAll && $item->level > 1 && !\in_array($item->parent_id, $path))
                        || ($start > 1 && !\in_array($item->tree[$start - 2], $path))) {
                        unset($items[$i]);

                        continue;
                    }

                    // Exclude item with menu item option set to exclude from menu modules
                    if (($itemParams->get('menu_show', 1) == 0) || in_array($item->parent_id, $hidden_parents)) {
                        $hidden_parents[] = $item->id;
                        unset($items[$i]);
                        continue;
                    }

                    $item->current = true;

                    foreach ($item->query as $key => $value) {
                        if (!isset($inputVars[$key]) || $inputVars[$key] !== $value) {
                            $item->current = false;
                            break;
                        }
                    }

                    $item->deeper = false;
                    $item->shallower = false;
                    $item->level_diff = 0;

                    if (isset($items[$lastitem])) {
                        $items[$lastitem]->deeper = ($item->level > $items[$lastitem]->level);
                        $items[$lastitem]->shallower = ($item->level < $items[$lastitem]->level);
                        $items[$lastitem]->level_diff = ($items[$lastitem]->level - $item->level);
                    }

                    $lastitem = $i;
                    $item->active = false;
                    $item->flink = $item->link;

                    // Reverted back for CMS version 2.5.6
                    switch ($item->type) {
                        case 'separator':
                            break;

                        case 'heading':
                            // No further action needed.
                            break;

                        case 'url':
                            if ((strpos($item->link, 'index.php?') === 0) && (strpos($item->link, 'Itemid=') === false)) {
                                // If this is an internal Joomla link, ensure the Itemid is set.
                                $item->flink = $item->link . '&Itemid=' . $item->id;
                            }
                            break;

                        case 'alias':
                            $item->flink = 'index.php?Itemid=' . $itemParams->get('aliasoptions');

                            // Get the language of the target menu item when site is multilingual
                            if (Multilanguage::isEnabled()) {
                                $newItem = Factory::getApplication()->getMenu()->getItem((int) $itemParams->get('aliasoptions'));

                                // Use language code if not set to ALL
                                if ($newItem != null && $newItem->language && $newItem->language !== '*') {
                                    $item->flink .= '&lang=' . $newItem->language;
                                }
                            }
                            break;

                        case 'superparent':
                            $item->flink = '#';
                            break;

                        default:
                            $item->flink = 'index.php?Itemid=' . $item->id;
                            break;
                    }

                    if ((strpos($item->flink, 'index.php?') !== false) && strcasecmp(substr($item->flink, 0, 4), 'http')) {
                        $item->flink = Route::_($item->flink, true, $itemParams->get('secure'));
                    } else {
                        $item->flink = Route::_($item->flink);
                    }

                    // We prevent the double encoding because for some reason the $item is shared for menu modules and we get double encoding
                    // when the cause of that is found the argument should be removed
                    $item->title = htmlspecialchars($item->title, ENT_COMPAT, 'UTF-8', false);
                    $item->menu_icon = htmlspecialchars($itemParams->get('menu_icon_css', ''), ENT_COMPAT, 'UTF-8', false);
                    $item->anchor_css = htmlspecialchars($itemParams->get('menu-anchor_css', ''), ENT_COMPAT, 'UTF-8', false);
                    $item->anchor_title = htmlspecialchars($itemParams->get('menu-anchor_title', ''), ENT_COMPAT, 'UTF-8', false);
                    $item->anchor_rel = htmlspecialchars($itemParams->get('menu-anchor_rel', ''), ENT_COMPAT, 'UTF-8', false);
                    $item->menu_image = htmlspecialchars($itemParams->get('menu_image', ''), ENT_COMPAT, 'UTF-8', false);
                    $item->menu_image_css = htmlspecialchars($itemParams->get('menu_image_css', ''), ENT_COMPAT, 'UTF-8', false);
                }

                if (isset($items[$lastitem])) {
                    $items[$lastitem]->deeper = (($start ?: 1) > $items[$lastitem]->level);
                    $items[$lastitem]->shallower = (($start ?: 1) < $items[$lastitem]->level);
                    $items[$lastitem]->level_diff = ($items[$lastitem]->level - ($start ?: 1));
                }
            }

            if (isset($key)) {
                $cache->store($items, $key);
            }
        }

        return $items;
    }


    public static function removeWhiteSpace($string)
    {
        $string = str_replace("\n", "", $string);
        $string = str_replace("\t", "", $string);
        $text = preg_replace('/([\s])\1+/', ' ', $string);
        $text = trim($text);
        return $text;
    }

    /**
     * Get Fontawesome classes.
     *
     * @return  object
     */
    public static function getFaFromIconClass($item)
    {

        //get fa classes
        $classes_array = array_map('trim', array_filter(explode(' ', $item->menu_icon)));
        $classes_no_fa = array();
        $classes_fa = array();
        foreach ($classes_array as $key => $class) {
            //check fa class
            if ($class == 'fas' ||
                $class == 'far' ||
                $class == 'fal' ||
                $class == 'fab' ||
                $class == 'fad' ||
                $class == 'fa-solid' ||
                $class == 'fa-regular' ||
                $class == 'fa-brands' ||
                $class == 'fa-light' ||
                $class == 'fa-thin' ||
                $class == 'fa-duotone'
            ) {
                $classes_fa[] = $class;
            } elseif (substr($class, 0, 3) === 'tm-') {
                $classes_fa[] = str_replace("tm-", "", $class);
            } elseif (substr($class, 0, 3) === 'fa-') {
                $classes_fa[] = $class;
            } else {
                $classes_no_fa[] = $class;
            }
        }

        $classes_fa = implode(' ', $classes_fa);

        $tmfa_array = array('fa' => $classes_fa, 'nofa' => $classes_no_fa);

        return $tmfa_array;
    }


    public static function getCaret($item, $params, $unique = '')
    {

        $tm_mobile_caret = new Registry();
        $tm_mobile_caret->loadString(json_encode($params->get('tm_mobile_caret')));

        $tm_desktop_caret = new Registry;
        $tm_desktop_caret->loadString(json_encode($params->get('tm_desktop_caret')));

        self::$caret_desktop_on = $tm_desktop_caret->get('tm_caret_open', 'fa-arrow-right');
        self::$caret_mobile_on = $tm_mobile_caret->get('tm_caret_open', 'fa-arrow-right');

        $tm_caret_mobile_style = $tm_mobile_caret->get('tm_caret_style', 'fas');
        $tm_caret_mobile_open = $tm_mobile_caret->get('tm_caret_open', '0');
        $tm_alias_mobile_caret = $tm_mobile_caret->get('tm_alias_caret', '0');

        $tm_caret_desktop_style = $tm_desktop_caret->get('tm_caret_style', 'fas');
        $tm_caret_desktop_open = $tm_desktop_caret->get('tm_caret_open', '0');
        $tm_alias_desktop_caret = $tm_desktop_caret->get('tm_alias_caret', '0');

        $tm_version = $params->get('tm_version', 4);

        $onclick = ($tm_version == 4) ? '' : 'onClick="tmOpenSubmenu(this)"';

        $rotate_mode_90deg_array = array('fa-plus', 'fa-plus-square', 'fa-plus-circle');

        //$tm_caret_desktop_style = 'fa';

        //ALIAS
        if ($item->type == 'alias') {
            //mobile
            $faclass_mobile = ($tm_alias_mobile_caret == '0' ? '' : $tm_caret_mobile_style . ' ' . $tm_alias_mobile_caret);
            $fa_a_class_mobile = ($tm_alias_mobile_caret == '0' ? ' no-tmflecha_mobile' : ' tmflecha_mobile');

            //check if the caret must rotate 45 or 90deg
            if (in_array($tm_alias_mobile_caret, $rotate_mode_90deg_array)) {
                $rotate_mobile_mode = ' tm_mobile_rotate_45';
            } else {
                $rotate_mobile_mode = ' tm_mobile_rotate_90';
            }

            //desktop
            $faclass_desktop = ($tm_alias_desktop_caret == '0' ? '' : $tm_caret_desktop_style . ' ' . $tm_alias_desktop_caret);
            $fa_a_class_desktop = ($tm_alias_desktop_caret == '0' ? ' no-tmflecha_desktop' : ' tmflecha_desktop');

            //check if the caret must rotate 45 or 90deg
            if (in_array($tm_alias_desktop_caret, $rotate_mode_90deg_array)) {
                $rotate_desktop_mode = ' tm_desktop_rotate_45';
            } else {
                $rotate_desktop_mode = ' tm_desktop_rotate_90';
            }

            $caret = '<a tabindex="0" '.$onclick.' role="button" aria-haspopup="true" aria-expanded="false" aria-label="' . Text::_('MOD_MENU_TOGGLE') . '" aria-controls="tmulid-' . $item->id . '"
				class="dxsubmenu ' . $fa_a_class_mobile . $fa_a_class_desktop . $rotate_mobile_mode . $rotate_desktop_mode . '" >';
            $caret .= ($faclass_mobile) ? '<i class="caret_mobile ' . $faclass_mobile . '"></i>' : '';
            $caret .= ($faclass_desktop) ? '<i class="caret_desktop ' . $faclass_desktop . '"></i>' : '';
            $caret .= '</a>';
        } else {
            //mobile
            $faclass_mobile = ($tm_caret_mobile_open == '0' ? '' : $tm_caret_mobile_style . ' ' . $tm_caret_mobile_open);
            $fa_a_class_mobile = ($tm_caret_mobile_open == '0' ? ' no-tmflecha_mobile' : ' tmflecha_mobile');

            //check if the caret must rotate 45 or 90deg
            if (in_array($tm_caret_mobile_open, $rotate_mode_90deg_array)) {
                $rotate_mobile_mode = ' tm_mobile_rotate_45';
            } else {
                $rotate_mobile_mode = ' tm_mobile_rotate_90';
            }

            //desktop
            $faclass_desktop = ($tm_caret_desktop_open == '0' ? '' : $tm_caret_desktop_style . ' ' . $tm_caret_desktop_open);
            $fa_a_class_desktop = ($tm_caret_desktop_open == '0' ? ' no-tmflecha_desktop' : ' tmflecha_desktop');

            //check if the caret must rotate 45 or 90deg
            if (in_array($tm_caret_desktop_open, $rotate_mode_90deg_array)) {
                $rotate_desktop_mode = ' tm_desktop_rotate_45';
            } else {
                $rotate_desktop_mode = ' tm_desktop_rotate_90';
            }
   
            $caret = '<a tabindex="0" '.$onclick.' role="button" aria-haspopup="true" aria-expanded="false" aria-label="' . Text::_('MOD_MENU_TOGGLE') . '" aria-controls="tmulid-' . $item->id . '"  class="dxsubmenu ' . $fa_a_class_mobile . $fa_a_class_desktop . $rotate_mobile_mode . $rotate_desktop_mode . '" >';
            $caret .= ($faclass_mobile) ? '<i class="caret_mobile ' . $faclass_mobile . '"></i>' : '';
            $caret .= ($faclass_desktop) ? '<i class="caret_desktop ' . $faclass_desktop . '"></i>' : '';
            $caret .= '</a>';
        }

        return $caret;
    }

        /**
     * Get base menu item.
     *
     * @param   \Joomla\Registry\Registry  &$params  The module options.
     *
     * @return  object
     *
     * @since    3.0.2
     */
    public static function getBase(&$params)
    {
        // Get base menu item from parameters
        if ($params->get('base')) {
            $base = Factory::getApplication()->getMenu()->getItem($params->get('base'));
        } else {
            $base = false;
        }

        // Use active menu item if no base found
        if (!$base) {
            $base = self::getActive($params);
        }

        return $base;
    }

    public static function getActive(&$params = '')
    {
        $menu = Factory::getApplication()->getMenu();

        return $menu->getActive() ?: self::getDefault();
    }

}
