<?php
/**
 * @package     Totalmenu
 * @subpackage  mod_totalmenu debug
 *
 * @copyright   (C) 2021 jtotal based on Joomla
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 */

namespace Jtotal\Module\Totalmenu\Site\Helper;

use Joomla\Filesystem\Folder;
use Joomla\Registry\Registry;
use Jtotal\Jtfw\Helper\JtScssPhp;

// phpcs:disable PSR1.Files.SideEffects
\defined('_JEXEC') or die;
// phpcs:enable PSR1.Files.SideEffects

abstract class TotalmenuProcessCss
{
    private static $customcode = array();
    private static $a_customcode = array();
    private static $c_customcode = array();

    //caret
    public static $caret_desktop_on;
    public static $caret_mobile_on;
    public static $caret_current_on;


    //GET CSS COMPLETE
    public static function getCSS($params, $moduleid)
    {
        $unique = htmlspecialchars($params->get('dx_uniquename').$moduleid);

        //mobile
        $tm_mobile_on = $params->get('tm_mobile_on', 1);
        //desktop
        $tm_desktop_on = $params->get('tm_desktop_on', 1);

        $desktopCss = ($tm_desktop_on) ? self::getDesktopCSS($params, $unique) : '';
        $mobileCss = ($tm_mobile_on) ? self::getMobileCSS($params, $unique) : '';




        $cache_path_abs = JPATH_SITE . '/cache/mod_totalmenu';
        if (!file_exists($cache_path_abs)) {
            Folder::create($cache_path_abs);
        }

        $cssOut = $mobileCss.$desktopCss;

        file_put_contents($cache_path_abs . '/' . $unique . '.css', $cssOut);

        return 'cache/mod_totalmenu/' . $unique . '.css';
    }

    public static function getDesktopCSS($params, $unique)
    {

        $cssOut = '';
        $tm_desktop_base = $params->get('tm_desktop_base', 'horizontal');
        $tm_desktop_style = $params->get('tm_desktop_style', 'custom');
        $tm_production = $params->get('tm_production', '0');
        $tm_sticky_desktop = $params->get('tm_sticky_desktop', '0');
        $tm_sticky_desktop_top = $params->get('tm_sticky_desktop_top', '0');
        $tm_zindex = $params->get('tm_zindex', '1');
        $tm_wrapper_max_width = $params->get('tm_wrapper_max_width', '600px');
        $tm_media_w = $params->get('tm_mobile_media_w', '990');

        $d_fadein = $params->get('tm_desktop_fadein', '300');

        $tm_debug = $params->get('tm_debug', '0');

        $desktop_exists = file_exists('cache/mod_totalmenu/' . $unique . '_desktop_' . $tm_desktop_style . '.css');

        if ($tm_production == 0 || $desktop_exists == false) {
            $scss = JtScssPhp::getScssphp();
            if (!$scss) {
                return false;
            }

            if ($tm_debug) {
                $scss->setOutputStyle(\ScssPhp\ScssPhp\OutputStyle::EXPANDED);
            } else {
                $scss->setOutputStyle(\ScssPhp\ScssPhp\OutputStyle::COMPRESSED);
            }

            $tm_imports = '';
            //desktop
            $tm_imports .= '@media (min-width: ' . $tm_media_w . 'px) {
		    	';
            $tm_imports .= '$tm_media_w:' . $tm_media_w . 'px;
		    	';

            //variables
            if ($tm_desktop_style == 'custom') {
                //from custom.php
                $tm_imports .= self::getVariables($params, 'desktop');
            }
            $tm_imports .= '#' . $unique . '{';

            $tm_imports .= '$fadein : ' . $d_fadein . 'ms;';

            if ($tm_sticky_desktop) {
                $tm_imports .= 'top:' . $tm_sticky_desktop_top . 'px !important;';
            }

            $tm_imports .= 'z-index:' . $tm_zindex . ';';

            //max width
            $tm_imports .= 'max-width:' . $tm_wrapper_max_width . ';';

            //PONER AL FINAL

            $tm_imports .= file_get_contents('modules/mod_totalmenu/scss/common.scss');
            $tm_imports .= file_get_contents('modules/mod_totalmenu/scss/common_desktop.scss');
            $tm_imports .= file_get_contents('modules/mod_totalmenu/scss/base/' . $tm_desktop_base . '.scss');
            $tm_imports .= file_get_contents('modules/mod_totalmenu/scss/styles/' . $tm_desktop_style . '_' . $tm_desktop_base . '.scss');

            //elements
            if ($tm_desktop_style == 'custom') {
                //from custom.php
                $tm_imports .= self::getElements($params->get('tm_desktop_custom', (object) array()), $params);
            }

            $tm_imports .= '}';
            $tm_imports .= '}';

            //$sass_comp_path = 'ScssPhp\ScssPhp\Formatter\Nested';

            $cssOut = $scss->compile($tm_imports);

            $cache_path_abs = JPATH_SITE . '/cache/mod_totalmenu';
            if (!file_exists($cache_path_abs)) {
                //mkdir(JPATH_CACHE.'/mod_totalmenu/', 0777, true);
                Folder::create($cache_path_abs);
            }

            file_put_contents($cache_path_abs . '/' . $unique . '_desktop_' . $tm_desktop_style . '.css', $cssOut);
            file_put_contents($cache_path_abs . '/' . $unique . '_desktop_' . $tm_desktop_style . '.scss', $tm_imports);

        }

        return $cssOut;
        return 'cache/mod_totalmenu/' . $unique . '_desktop_' . $tm_desktop_style . '.css';
    }

    public static function getMobileCSS($params, $unique)
    {
        $cssOut = '';
        $tm_mobile_base = $params->get('tm_mobile_base', 'vertical');
        $tm_mobile_style = $params->get('tm_mobile_style', 'custom');
        $tm_production = $params->get('tm_production', '0');
        $tm_sticky_mobile = $params->get('tm_sticky_mobile', '0');
        $tm_sticky_mobile_top = $params->get('tm_sticky_mobile_top', '0');

        $tm_zindex = $params->get('tm_zindex', '1');
        $tm_wrapper_max_width = $params->get('tm_wrapper_mobile_max_width', '600px');
        $tm_media_w = $params->get('tm_mobile_media_w', '990');

        $m_fadein = $params->get('tm_mobile_fadein', '300');
        $tm_debug = $params->get('tm_debug', '0');

        $mobile_exists = file_exists('cache/mod_totalmenu/' . $unique . '_mobile_' . $tm_mobile_style . '.css');

        if ($tm_production == 0 || $mobile_exists == false) {
            $scss = JTscssphp::getScssphp();
            if (!$scss) {
                return;
            }

            if ($tm_debug) {
                $scss->setOutputStyle(\ScssPhp\ScssPhp\OutputStyle::EXPANDED);
            } else {
                //$scss->setFormatter('ScssPhp\ScssPhp\Formatter\Compressed');
                $scss->setOutputStyle(\ScssPhp\ScssPhp\OutputStyle::COMPRESSED);
            }

            $tm_imports = '';
            //mobile
            $tm_imports .= '@media (max-width: ' . ($tm_media_w - 1) . 'px) {
		    	';
            $tm_imports .= '$tm_media_w:' . $tm_media_w . 'px;
		    	';

            //variables
            if ($tm_mobile_style == 'custom') {
                //from custom.php
                $tm_imports .= self::getVariables($params, 'mobile');
            }
            $tm_imports .= '#' . $unique . '{';

            $tm_imports .= '$fadein : ' . $m_fadein . 'ms;';

            if ($tm_sticky_mobile) {
                $tm_imports .= 'top:' . $tm_sticky_mobile_top . 'px !important;';
            }

            $tm_imports .= 'z-index:' . $tm_zindex . ';';

            //max width
            $tm_imports .= 'max-width:' . $tm_wrapper_max_width . ';';

            $tm_imports .= file_get_contents('modules/mod_totalmenu/scss/common.scss');
            $tm_imports .= file_get_contents('modules/mod_totalmenu/scss/common_mobile.scss');
            $tm_imports .= file_get_contents('modules/mod_totalmenu/scss/base/' . $tm_mobile_base . '.scss');
            $tm_imports .= file_get_contents('modules/mod_totalmenu/scss/styles/' . $tm_mobile_style . '_' . $tm_mobile_base . '.scss');

            //custom lvl0
            //why?
            /*     $tm_imports .= '
            ul.tmlevel-0 {';
            $tm_imports .= self::$customcode['lvl0'];

            $tm_imports .= '>li:hover{';
            $tm_imports .= self::$a_customcode['lvl0'];
            $tm_imports .= '}';

            $tm_imports .= '>li.current{';
            $tm_imports .= self::$c_customcode['lvl0'];
            $tm_imports .= '}';

            $tm_imports .= '}'; */

            //elements
            if ($tm_mobile_style == 'custom') {
                //from custom.php
                $tm_imports .= self::getElements($params->get('tm_mobile_custom', (object) array()), $params);
            }

            $tm_imports .= '}';
            $tm_imports .= '}';

            //sass compress and output
            //$sass_comp_path = 'ScssPhp\ScssPhp\Formatter\Nested';
            $cssOut = $scss->compile($tm_imports);

            $cache_path_abs = JPATH_SITE . '/cache/mod_totalmenu';
            if (!file_exists($cache_path_abs)) {
                //mkdir(JPATH_CACHE.'/mod_totalmenu/', 0777, true);
                Folder::create($cache_path_abs);
            }

            file_put_contents($cache_path_abs . '/' . $unique . '_mobile_' . $tm_mobile_style . '.css', $cssOut);
            if ($tm_debug) {
                file_put_contents($cache_path_abs . '/' . $unique . '_mobile_' . $tm_mobile_style . '.scss', $tm_imports);
            }
        }
        return $cssOut;
        return 'cache/mod_totalmenu/' . $unique . '_mobile_' . $tm_mobile_style . '.css';
    }

    private static function getElements($custom, $params)
    {

        $atribs_obj = $custom;

        $custom_css = '';
        $lvl = 0;
        foreach ($atribs_obj as $key => $prop) {


            //CSS ELEMENT
            if (!is_object($prop)) {
                $prop = json_decode($prop);
            }
            $prop = new Registry($prop->tmcustom);

            $custom_css .= '

			ul.tmlevel-' . $lvl . ' {';

            //ul border radius
            if ($prop->get('tm_ul_on', 0)) {
                $custom_css .= 'border-radius: $' . ($key) . '_ul_border_radius;';
                $custom_css .= 'border-style: solid;';
                $custom_css .= 'border-width: $' . ($key) . '_ul_border_width;';
                $custom_css .= 'border-color: $' . ($key) . '_ul_border_color;';
                $custom_css .= 'box-shadow: $' . ($key) . '_ul_box_shadow;';
            }

            //code
            if ($prop->get('tm_code_on', 0)) {
                $custom_css .= self::$customcode[$key];
            }

            //
            // LI
            //
            $custom_css .= '
				>li{';

            //border radius
            if ($prop->get('tm_ul_on', 0)) {
                $custom_css .= '&:first-child{
							>div.dxgroup{
								border-radius: $' . $key . '_ul_border_radius_first_li;
							}
						}
						&:last-child{
							border-bottom: none;
							>div.dxgroup{
								border-radius: $' . $key . '_ul_border_radius_last_li;
							}
						}';
            }

            //
            // DX GROUP
            //
            $custom_css .= '
					>div.dxgroup{
						';

            //border radius
            if ($prop->get('tm_border_radius_on', 0)) {
                $custom_css .= 'border-radius: $' . ($key) . '_border_radius;';
            }

            if ($prop->get('tm_background_on', 0)) {
                $custom_css .= '
							background: $' . $key . '_bg;
							';
            }
            if ($prop->get('tm_border_width_on', 0)) {
                $custom_css .= '
							border-width: $' . ($key) . '_border_width;
							';
            }

            if ($prop->get('tm_border_color_on', 0)) {
                $custom_css .= '
							border-color: $' . ($key) . '_border_color;
							';
            }

            //
            // A
            //
            $custom_css .= '
						a{';

            if ($prop->get('tm_color_on', 0)) {
                $custom_css .= '
								color: $' . ($key) . '_color; ';
            }
            if ($prop->get('tm_size_on', 0)) {
                $custom_css .= '
								font-size: $' . ($key) . '_size; ';
                $custom_css .= '
								line-height: $' . ($key) . '_size; ';
            }

            if ($prop->get('tm_padding_on', 0)) {
                $custom_css .= '
								padding: $' . ($key) . '_padding; ';
            }

            $custom_css .= '
						}';
            $custom_css .= '
					}';

            ////////////////////////////////////////////////////////////
            // LI hover (active)
            //
            $custom_css .= '
					&:hover{';

            //code
            if ($prop->get('tm_code_on', 0)) {
                $custom_css .= self::$a_customcode[$key];
            }

            //
            // dx group
            //
            $custom_css .= '
						>div.dxgroup{
							';

            if ($prop->get('tm_background_on', 0)) {
                $custom_css .= '
							background: $' . $key . '_a_bg;
							';
            }
            if ($prop->get('tm_border_width_on', 0)) {
                $custom_css .= '
							border-width: $' . ($key) . '_a_border_width;
							';
            }

            if ($prop->get('tm_border_color_on', 0)) {
                $custom_css .= '
							border-color: $' . ($key) . '_a_border_color;
							';
            }

            //
            // A
            //
            $custom_css .= '
							a{';

            if ($prop->get('tm_color_on', 0)) {
                $custom_css .= '
								color: $' . ($key) . '_a_color; ';
            }
            if ($prop->get('tm_size_on', 0)) {
                $custom_css .= '
								font-size: $' . ($key) . '_a_size; ';
                $custom_css .= '
								line-height: $' . ($key) . '_a_size; ';
            }
            if ($prop->get('tm_padding_on', 0)) {
                $custom_css .= '
								padding: $' . ($key) . '_a_padding; ';
            }

            $custom_css .= '
							}';
            $custom_css .= '
						}';
            $custom_css .= '
					}';

            ////////////////////////////////////////////////////////////
            // LI current
            ////////////////////////////////////////////////////////////
            $custom_css .= '
					&.current,&.tm_open{';

            //code
            if ($prop->get('tm_code_on', 0)) {
                $custom_css .= self::$a_customcode[$key];
            }

            //
            // dx group
            //
            $custom_css .= '
						>div.dxgroup{
							';
            //border radius
            if ($prop->get('tm_border_radius_on', 0)) {
                $custom_css .= 'border-radius: $' . ($key) . '_c_border_radius;';
            }
            if ($prop->get('tm_background_on', 0)) {
                $custom_css .= '
								background: $' . $key . '_c_bg;
								';
            }
            if ($prop->get('tm_border_width_on', 0)) {
                $custom_css .= '
								border-width: $' . ($key) . '_c_border_width;
								';
            }

            if ($prop->get('tm_border_color_on', 0)) {
                $custom_css .= '
								border-color: $' . ($key) . '_c_border_color;
								';
            }

            //
            // A
            //
            $custom_css .= '
							a{';

            if ($prop->get('tm_color_on', 0)) {
                $custom_css .= '
								color: $' . ($key) . '_c_color; ';
            }
            if ($prop->get('tm_size_on', 0)) {
                $custom_css .= '
								font-size: $' . ($key) . '_c_size; ';
                $custom_css .= '
								line-height: $' . ($key) . '_c_size; ';
            }
            if ($prop->get('tm_padding_on', 0)) {
                $custom_css .= '
								padding: $' . ($key) . '_c_padding; ';
            }
            //close a
            $custom_css .= '
							}';
            //close dxgroup
            $custom_css .= '
						}';
            //close current
            $custom_css .= '
					}';

            //close LI
            $custom_css .= '
				}';

            //close UL
            $custom_css .= '
			}';

            $lvl++;
        }

        return $custom_css;
    }

    /******************************************************/
    /************            VARIABLES            *********/
    /******************************************************/

    private static function getVariables($params, $context = '')
    {

        //carets
        $tm_mobile_caret = new Registry;
        $tm_mobile_caret->loadString(json_encode($params->get('tm_mobile_caret')));
        self::$caret_mobile_on = $tm_mobile_caret->get('tm_caret_open', 'fa-arrow-right');

        $tm_desktop_caret = new Registry;
        $tm_desktop_caret->loadString(json_encode($params->get('tm_desktop_caret')));
        self::$caret_desktop_on = $tm_desktop_caret->get('tm_caret_open', 'fa-arrow-right');

        if ($context == 'mobile') {
            $custom = $params->get('tm_mobile_custom', (object) array());
            self::$caret_current_on = self::$caret_mobile_on;
        } else {
            $custom = $params->get('tm_desktop_custom', (object) array());
            self::$caret_current_on = self::$caret_desktop_on;
        }

        $desktop_base = $params->get('tm_desktop_base', 'horizontal');
        $mobile_base = $params->get('tm_mobile_base', 'horizontal');

        //$atribs_array = json_decode($custom);
        $atribs_obj = $custom;

        $custom_css = '';

        //prevent errors
        $custom_css = '
				$lvl0_bg: transparent !default;
				$lvl0_bg_open:transparent !default;
				$lvl0_bg_current:transparent !default;
				$lvl0_size:1rem !default;
				$lvl0_color:#333 !default;
				$lvl0_current:#111 !default;
				$lvl0_border_size:0 !default;
				$lvl0_active_border_size:0 !default;
				$lvl0_border_color:transparent !default;
				$lvl0_active_border_color:transparent !default;
				$lvl0_padding:10px 6px 6px 6px !default;
				$lvl0_active_padding:10px 6px 6px 6px !default;
				$lvl0_br:0px 0px 0px 0px !default;
				$lvl0_a_br:0px 0px 0px 0px !default;

			';

        //self::$customcode['lvl'.$key] = $code;
        //self::$a_customcode['lvl'.$key] = $a_code;
        //self::$c_customcode['lvl'.$key] = $c_code;

        foreach ($atribs_obj as $key => $table) {
            //KEY = tm_desktop_custom0, tm_desktop_custom1, etc
            foreach ($table as $propkey => $prop) {
                if (!is_object($prop)) {
                    $prop = json_decode($prop);
                }
                $prop = new Registry($prop);

                //var_dump($prop);

                //backgroud
                if ($prop->get('tm_background_on', 0)) {
                    $custom_css .= '$' . $key . '_bg:' . $prop->get('tm_background', '') . ';';
                    $custom_css .= '$' . $key . '_a_bg:' . $prop->get('a_tm_background', '') . ';';
                    $custom_css .= '$' . $key . '_c_bg:' . $prop->get('c_tm_background', '') . ';';
                }

                //color
                if ($prop->get('tm_color_on', 0)) {
                    $custom_css .= '$' . $key . '_color:' . $prop->get('tm_color', '') . ';';
                    $custom_css .= '$' . $key . '_a_color:' . $prop->get('a_tm_color', '') . ';';
                    $custom_css .= '$' . $key . '_c_color:' . $prop->get('c_tm_color', '') . ';';
                }

                //size
                if ($prop->get('tm_size_on', 0)) {
                    $custom_css .= '$' . $key . '_size:' . $prop->get('tm_size', '') . 'px;';
                    $custom_css .= '$' . $key . '_a_size:' . $prop->get('a_tm_size', '') . 'px;';
                    $custom_css .= '$' . $key . '_c_size:' . $prop->get('c_tm_size', '') . 'px;';
                }

                //BORDER width
                if ($prop->get('tm_border_width_on', 0)) {
                    $b_width = str_replace(',', 'px ', $prop->get('tm_border_width', '0,0,0,0')) . 'px';
                    $a_b_width = str_replace(',', 'px ', $prop->get('a_tm_border_width', '0,0,0,0')) . 'px';
                    $c_b_width = str_replace(',', 'px ', $prop->get('c_tm_border_width', '0,0,0,0')) . 'px';

                    $custom_css .= '$' . $key . '_border_width:' . $b_width . ';';
                    $custom_css .= '$' . $key . '_a_border_width:' . $a_b_width . ';';
                    $custom_css .= '$' . $key . '_c_border_width:' . $c_b_width . ';';
                }

                //BORDER color
                if ($prop->get('tm_border_color_on', 0)) {
                    $b_color = $prop->get('tm_border_color', 'rgba(0,0,0,0');
                    $a_b_color = $prop->get('a_tm_border_color', 'rgba(0,0,0,0');
                    $c_b_color = $prop->get('c_tm_border_color', 'rgba(0,0,0,0');

                    $custom_css .= '$' . $key . '_border_color:' . $b_color . ';';
                    $custom_css .= '$' . $key . '_a_border_color:' . $a_b_color . ';';
                    $custom_css .= '$' . $key . '_c_border_color:' . $c_b_color . ';';
                }

                //PADDING
                if ($prop->get('tm_padding_on', 0)) {
                    $padding = str_replace(',', 'px ', $prop->get('tm_padding', '0,0,0,0')) . 'px';
                    $a_padding = str_replace(',', 'px ', $prop->get('a_tm_padding', '0,0,0,0')) . 'px';
                    $c_padding = str_replace(',', 'px ', $prop->get('c_tm_padding', '0,0,0,0')) . 'px';

                    $custom_css .= '$' . $key . '_padding:' . $padding . ';';
                    $custom_css .= '$' . $key . '_a_padding:' . $a_padding . ';';
                    $custom_css .= '$' . $key . '_c_padding:' . $c_padding . ';';
                }

                //var_dump($prop);

                //BORDER radius
                if ($prop->get('tm_border_radius_on', 0)) {
                    $b_radius = str_replace(',', 'px ', $prop->get('tm_border_radius', '0,0,0,0')) . 'px';
                    $b_radius_array = explode(',', $prop->get('tm_border_radius', '0,0,0,0'));

                    $a_b_radius = str_replace(',', 'px ', $prop->get('a_tm_border_radius', '0,0,0,0')) . 'px';
                    $a_b_radius_array = explode(',', $prop->get('a_tm_border_radius', '0,0,0,0'));

                    $c_b_radius = str_replace(',', 'px ', $prop->get('c_tm_border_radius', '0,0,0,0')) . 'px';
                    $c_b_radius_array = explode(',', $prop->get('c_tm_border_radius', '0,0,0,0'));

                    $custom_css .= '$' . $key . '_border_radius:' . $b_radius . ';';
                    //$custom_css .= '$'.$key.'_border_radius_first_li:'.$b_radius_array[0].'px '.$b_radius_array[1].'px 0px 0px;';
                    //$custom_css .= '$'.$key.'_border_radius_last_li: 0px 0px '.$b_radius_array[2].'px '.$b_radius_array[3].'px;';

                    $custom_css .= '$' . $key . '_a_border_radius:' . $a_b_radius . ';';
                    //$custom_css .= '$'.$key.'_a_border_radius_first_li:'.$a_b_radius_array[0].'px '.$a_b_radius_array[1].'px 0px 0px;';
                    //$custom_css .= '$'.$key.'_a_border_radius_last_li: 0px 0px '.$a_b_radius_array[2].'px '.$a_b_radius_array[3].'px;';

                    $custom_css .= '$' . $key . '_c_border_radius:' . $c_b_radius . ';';
                    //$custom_css .= '$'.$key.'_c_border_radius_first_li:'.$c_b_radius_array[0].'px '.$c_b_radius_array[1].'px 0px 0px;';
                    //$custom_css .= '$'.$key.'_c_border_radius_last_li: 0px 0px '.$c_b_radius_array[2].'px '.$c_b_radius_array[3].'px;';
                }

                //CODE
                if ($prop->get('tm_code_on', 0)) {
                    $code = $prop->get('tm_code', '');
                    $a_code = $prop->get('a_tm_code', '');
                    $c_code = $prop->get('c_tm_code', '');

                    self::$customcode[$key] = $code;
                    self::$a_customcode[$key] = $a_code;
                    self::$c_customcode[$key] = $c_code;
                }

                //UL WRAPPER
                if ($prop->get('tm_ul_on', 0)) {
                    $ul_box_shadow = str_replace(',', 'px ', $prop->get('tm_ul_box_shadow', '0,0,0,0')) . 'px';
                    $ul_box_shadow_color = $prop->get('tm_ul_box_shadow_color', 'rgba(0,0,0,0)');
                    $custom_css .= '$' . $key . '_ul_box_shadow:' . $ul_box_shadow . ' ' . $ul_box_shadow_color . ';';

                    $ul_b_color = $prop->get('tm_ul_border_color', 'rgba(0,0,0,0');
                    $custom_css .= '$' . $key . '_ul_border_color:' . $ul_b_color . ';';

                    $ul_b_width = str_replace(',', 'px ', $prop->get('tm_ul_border_width', '0,0,0,0')) . 'px';
                    $ul_b_width_array = explode(',', $prop->get('tm_ul_border_width', '0,0,0,0'));
                    $custom_css .= '$' . $key . '_ul_border_width:' . $ul_b_width . ';';

                    //BORDER RADIUS
                    $ul_b_radius = str_replace(',', 'px ', $prop->get('tm_ul_border_radius', '0,0,0,0')) . 'px';
                    $ul_b_radius_array = explode(',', $prop->get('tm_ul_border_radius', '0,0,0,0'));

                    $ul_b_radius_final = array(
                        ($ul_b_radius_array[0] - $ul_b_width_array[0] >= 0) ? $ul_b_radius_array[0] - $ul_b_width_array[0] : 0,
                        ($ul_b_radius_array[1] - $ul_b_width_array[1] >= 0) ? $ul_b_radius_array[1] - $ul_b_width_array[1] : 0,
                        ($ul_b_radius_array[2] - $ul_b_width_array[2] >= 0) ? $ul_b_radius_array[2] - $ul_b_width_array[2] : 0,
                        ($ul_b_radius_array[3] - $ul_b_width_array[3] >= 0) ? $ul_b_radius_array[3] - $ul_b_width_array[3] : 0,
                    );

                    //BORDER RADIUS child
                    //si es el primer elemento de desktop configured in horiz
                    if ($context == 'desktop' && $desktop_base == 'horizontal' && $key == 'tm_desktop_custom0') {
                        $custom_css .= '$' . $key . '_ul_border_radius:' . $ul_b_radius . ';';
                        $custom_css .= '$' . $key . '_ul_border_radius_first_li:' . $ul_b_radius_final[0] . 'px 0px 0px ' . $ul_b_radius_final[3] . 'px;';
                        $custom_css .= '$' . $key . '_ul_border_radius_last_li: 0px ' . $ul_b_radius_final[1] . 'px ' . $ul_b_radius_final[2] . 'px 0px;';
                    } else {
                        $custom_css .= '$' . $key . '_ul_border_radius:' . $ul_b_radius . ';';
                        $custom_css .= '$' . $key . '_ul_border_radius_first_li:' . $ul_b_radius_final[0] . 'px ' . $ul_b_radius_final[1] . 'px 0px 0px;';
                        $custom_css .= '$' . $key . '_ul_border_radius_last_li: 0px 0px ' . $ul_b_radius_final[2] . 'px ' . $ul_b_radius_final[3] . 'px;';
                    }
                }
            }
        }

        return $custom_css;
    }
}
