<?php

/**
 * @package     Joomla.Site
 * @subpackage  mod_total_menu
 *
 * @copyright   Copyright (C) 2005 - 2019 jtotal, Inc. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 */

defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Helper\ModuleHelper;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Layout\LayoutHelper;
use Joomla\Registry\Registry;
use Jtotal\Module\Totalmenu\Site\Helper\TotalmenuHelper;

if (is_file(JPATH_PLUGINS . '/system/jtframework/autoload.php')) {
    require_once JPATH_PLUGINS . '/system/jtframework/autoload.php';
} else {
    $app = Factory::getApplication();
    $app->enqueueMessage(Text::_('JT_FW_NOT_FOUND'), 'error');
    return;
}

if ($tm_debug) {
    $msg = 'totalmenu "' . $modulename . '" classes:' . $tm_classes;
    //$document->addScriptDeclaration("console.log('{$msg}')");
}

?>

<div id="tm_original_<?Php echo $modulename; ?>" class="tm_originalpos_<?Php echo $modulename; ?>"></div>

<div id="<?Php echo $modulename; ?>" 
    class="<?Php echo $tm_classes; ?>" 
    data-moduleid="<?Php echo $module->id; ?>" 
    data-detection="<?Php echo $detection; ?>" 
    data-jmobile="<?Php echo $tm_jmobile; ?>" 
    data-cssdesktop="<?php echo $css_file_desktop_exists ? 'true' : 'false'; ?>" 
    data-cssmobile="<?php echo $css_file_mobile_exists ? 'true' : 'false'; ?>"
    data-mobile_dir="<?php echo     $tm_mobile_base ?>"
    data-desk_dir="<?php echo $tm_desktop_base ?>"
    data-media-w="<?php echo $tm_mobile_media_w ?>"
>

    <nav class="tm_navbar_wrapper <?php echo $fixedtop; ?>" role="navigation">

        <?php if ($dx_brand_on) {
            $bs_classes = 'col-lg-10 col-md-10 col-sm-8 col-xs-12';
        } else {
            $bs_classes = 'col-xs-12';
        }

        $dx_brand_master = '';

        //only mobile
        if ($dx_brand_on == '2') {
            $dx_brand_master .= ' tm_brand_only_mobile';
        } elseif ($dx_brand_on == '3') {
            $dx_brand_master .= ' tm_brand_only_desktop';
        }

        $dx_navbar_master = '';

        if ($dx_brand_on) : ?>
            <?php
                $final_brand = HTMLHelper::image($dx_brand, $dx_brand_alt, array('class' => 'img-fluid ' . $dx_brand_class, 'alt' => $dx_brand_alt));
            ?>
            

            <div class="tm_navbar-header <?php echo $dx_brand_master . ' ' . $tm_position_mobile; ?>" style="max-width: <?php echo $dx_brand_max_w; ?>px">
                <?php
                if ($dx_brand_on) {
                    echo '<a draggable="false" class="navbar-brand tm_navbar-brand" href="' . $dx_brand_link . '">';
                        echo $final_brand;
                    echo '</a>';
                }
                ?>

                <?php if ($tm_jmobile) : ?>
                    <button type="button" class="tm_navbar-toggle <?php echo $tm_collapsed . ' ' . $tm_show_collapsed; ?>" data-target=".<?php echo $modulename; ?>" aria-label="menu" aria-expanded="false" aria-controls="tm_navbar"><i class="fa fas fa-plus-circle" aria-hidden="true"></i></button>
                <?php endif; ?>
            </div>

        <?php endif; ?>


        <div class="tm_navbar <?php echo $tm_collapsed; ?> navbar-collapse <?php echo $modulename . $dx_navbar_master; ?>">

            <?php

            if ($tm_desktop_show_parent) {
                $item_parent = $menu_parent;

                $item_parent->anchor_css = $menu_parent->getParams()->get('menu-anchor_css', '');

                $getFa  = TotalmenuHelper::getFaFromIconClass($item);
                $classes_fa = $getFa['fa'];
                $fa_icon = $classes_fa ? '<i class="tm_fa_menu ' . $classes_fa . '"></i>' : '';

                echo '<h3 class="tm_show_parent_desktop">' . $fa_icon . ' ' . $menu_parent->title . '</h3>';
            }
            ?>

            <ul class=" totalmenu tmlevel-0
            menu<?php echo $class_sfx;
                echo $fixedtop;
                echo ' ' . $tm_justify; ?>" 
                <?php
                    $tag = '';

                    if ($params->get('tag_id') != null) {
                        $tag = $params->get('tag_id') . '';
                        echo ' id="' . $tag . '"';
                    }

                    ?>
                role="menu">


                <?php foreach ($lists as $p => $list) : ?>
                    <?php foreach ($list as $i => &$item) {
                        $itemParams = new Registry();
                        if ($params->get('tmmode', 'menu') == 'menu') {
                            $itemParams = $item->getParams();
                        }

                        $getFa = TotalmenuHelper::getFaFromIconClass($item);

                        $classes_fa = $getFa['fa'];
                        $fa_icon = $classes_fa ? '<i class="tm_fa_menu ' . $classes_fa . '"></i>' : '';

                        $classes_no_fa = $getFa['nofa'];
                        $custom_classes_from_menu = implode(' ', $classes_no_fa);

                        $class = 'item-' . $item->id;

                        if ($item->id == $active_id) {
                            $class .= ' current mobileopen';
                        }

                        if (in_array($item->id, $path)) {
                            $class .= ' active mobileopen';
                        } elseif ($item->type == 'alias' || $item->type == 'superparent' || $item->type == 'separator') {

                            //deprecated?
                            $aliasToId = $itemParams->get('aliasoptions');

                            if (count($path) > 0 && $aliasToId == $path[count($path) - 1]) {
                                $class .= ' active';
                            } elseif (in_array($aliasToId, $path)) {
                                $class .= ' alias-parent-active';
                            }
                        }

                        if ($item->type == 'component') {

                            if (in_array($item->id, $path) && $params->get('tm_startopened_desktop', 0) == '1') {
                                $class .= ' current active desktop_li_start_open';
                            }
                            if (in_array($item->id, $path) && $params->get('tm_startopened_mobile', 0) == '1') {
                                $class .= ' current active mobile_li_start_open';
                            }
                        }

                        if ($item->type == 'alias' && $tm_parent_element == 'alias') {
                            $class .= ' dx_alias';
                        }

                        if ($item->type == 'superparent') {
                            $class .= ' dx_alias';
                        }

                        if ($item->type == 'separator') {
                            $class .= ' dx_alias';
                        }

                        if ($item->deeper) {
                            $class .= ' deeper lilevel-' . $item->level . ' tm_closed tm_closed_done';
                        }

                        if ($item->parent) {
                            $class .= ' parent tmdropdown ';
                        }

                        //CUSTOM COLORS FROM MENU
                        $class .= ' ' . $item->anchor_css;
   
             

                        if (!empty($class)) {
                            $class = ' class="' . trim($class) . '"';
                        }

                        echo '<li' . $class . ' role="menuitem">';

                        // Render the menu item.

                        switch ($item->type):
                            case 'separator':
                                if ($tm_parent_element == 'separator') {
                                    require ModuleHelper::getLayoutPath('mod_totalmenu', 'default_component');
                                } else {
                                    require ModuleHelper::getLayoutPath('mod_totalmenu', 'default_' . $item->type);
                                }
                                break;
                            case 'url':
                                require ModuleHelper::getLayoutPath('mod_totalmenu', 'default_' . $item->type);
                                break;
                            case 'component':
                                require ModuleHelper::getLayoutPath('mod_totalmenu', 'default_' . $item->type);
                                break;
                            case 'heading':
                                require ModuleHelper::getLayoutPath('mod_totalmenu', 'default_' . $item->type);
                                break;
                            default:
                                require ModuleHelper::getLayoutPath('mod_totalmenu', 'default_component');
                                break;
                        endswitch;

                        // The next item is deeper.
                        if ($item->deeper) {
                            //echo '<div class="tm_subwrapper">';
                            echo '<ul class="tmdropdown-menu nav-child tm_loading tmlevel-' . $item->level . '" style="display:none;" role="menu">';
                        } elseif ($item->shallower) {
                            // The next item is shallower.
                            echo '</li>';
                            echo str_repeat('</ul></li>', $item->level_diff);
                            //echo str_repeat('</ul></li>', $item->level_diff);
                        } else {
                            // The next item is on the same level.
                            echo '</li>';
                        }
                    }
                    ?>

                <?php endforeach; ?>
            </ul>
        </div>
    </nav>
    <?php

    if (!$tm_production) {
        echo '<div class="tm_dev_mode" style="left:' . rand(5, 100) . 'px; 	background-color:rgb(' . rand(100, 255) . ',' . rand(100, 255) . ',' . rand(100, 255) . ');">
            <img src="media/mod_totalmenu/assets/images/laptop-code-solid.svg" style="width:32px; padding:5px;"/>';
        echo '<div class="tm_dev_mode_desktop">
                <span draggable="false" title="' . $unique . ' in development mode"><i class="fa fa-desktop"></i><i class="fa fa-cog"></i></span>
                ' . $unique . ' in development mode</div>';

        echo '</div>';
    }

    ?>

</div>